from typing import List, Optional
from .ResponseBase import ResponseBase

class GlobalAddressVerificationResponse(ResponseBase):
    def __init__(self, 
                 version = "", 
                 transmission_reference = "", 
                 transmission_results = "", 
                 total_records = "", 
                 records: list = None):
        self.version = version
        self.transmission_reference = transmission_reference
        self.transmission_results = transmission_results
        self.total_records = total_records
        self.records = records if records is not None else []

    @classmethod
    def populate_from_dict(cls, data: dict):
        records = [GlobalAddressVerificationRecord.from_dict(record) for record in data.get("Records", [])]
        cls.data = data
        return cls(
            version=data.get("Version", ""),
            transmission_reference=data.get("TransmissionReference", ""),
            transmission_results=data.get("TransmissionResults", ""),
            total_records=data.get("TotalRecords", ""),
            records=records
        )
    
    # Setters
    def set_version(self, version):
        self.version = version

    def set_transmission_reference(self, transmission_reference):
        self.transmission_reference = transmission_reference

    def set_transmission_results(self, transmission_results):
        self.transmission_results = transmission_results

    def set_total_records(self, total_records):
        self.total_records = total_records

    # Getters
    def get_version(self):
        return self.version or ""

    def get_transmission_reference(self):
        return self.transmission_reference or ""

    def get_transmission_results(self):
        return self.transmission_results or ""

    def get_total_records(self):
        return self.total_records or ""
    

class GlobalAddressVerificationRecord(ResponseBase):
    def __init__( self, record_id="", results="", formatted_address="", organization="", address_line_1="", address_line_2="", address_line_3="", 
                 address_line_4="", address_line_5="", address_line_6="", address_line_7="", address_line_8="", sub_premises="", double_dependent_locality="", 
                 dependent_locality="", locality="", sub_administrative_area="", administrative_area="", postal_code="", postal_code_type="", 
                 address_type="", address_key="", sub_national_area="", country_name="", country_iso3166_1_alpha2="", country_iso3166_1_alpha3="", 
                 country_iso3166_1_numeric="", country_subdivision_code="", thoroughfare="", thoroughfare_pre_direction="", thoroughfare_leading_type="", 
                 thoroughfare_name="", thoroughfare_trailing_type="", thoroughfare_post_direction="", dependent_thoroughfare="", 
                 dependent_thoroughfare_pre_direction="", dependent_thoroughfare_leading_type="", dependent_thoroughfare_name="", 
                 dependent_thoroughfare_trailing_type="", dependent_thoroughfare_post_direction="", building="", premises_type="", 
                 premises_number="", sub_premises_type="", sub_premises_number="", post_box="", latitude="", longitude="", delivery_indicator="", 
                 melissa_address_key="", melissa_address_key_base="", post_office_location="", sub_premise_level="", sub_premise_level_type="", 
                 sub_premise_level_number="", sub_building="", sub_building_type="", sub_building_number="", utc="", dst="", delivery_point_suffix="", 
                 census_key="", extras = None
    ):
        self.record_id = record_id
        self.results = results
        self.formatted_address = formatted_address
        self.organization = organization
        self.address_line_1 = address_line_1
        self.address_line_2 = address_line_2
        self.address_line_3 = address_line_3
        self.address_line_4 = address_line_4
        self.address_line_5 = address_line_5
        self.address_line_6 = address_line_6
        self.address_line_7 = address_line_7
        self.address_line_8 = address_line_8
        self.sub_premises = sub_premises
        self.double_dependent_locality = double_dependent_locality
        self.dependent_locality = dependent_locality
        self.locality = locality
        self.sub_administrative_area = sub_administrative_area
        self.administrative_area = administrative_area
        self.postal_code = postal_code
        self.postal_code_type = postal_code_type
        self.address_type = address_type
        self.address_key = address_key
        self.sub_national_area = sub_national_area
        self.country_name = country_name
        self.country_iso3166_1_alpha2 = country_iso3166_1_alpha2
        self.country_iso3166_1_alpha3 = country_iso3166_1_alpha3
        self.country_iso3166_1_numeric = country_iso3166_1_numeric
        self.country_subdivision_code = country_subdivision_code
        self.thoroughfare = thoroughfare
        self.thoroughfare_pre_direction = thoroughfare_pre_direction
        self.thoroughfare_leading_type = thoroughfare_leading_type
        self.thoroughfare_name = thoroughfare_name
        self.thoroughfare_trailing_type = thoroughfare_trailing_type
        self.thoroughfare_post_direction = thoroughfare_post_direction
        self.dependent_thoroughfare = dependent_thoroughfare
        self.dependent_thoroughfare_pre_direction = dependent_thoroughfare_pre_direction
        self.dependent_thoroughfare_leading_type = dependent_thoroughfare_leading_type
        self.dependent_thoroughfare_name = dependent_thoroughfare_name
        self.dependent_thoroughfare_trailing_type = dependent_thoroughfare_trailing_type
        self.dependent_thoroughfare_post_direction = dependent_thoroughfare_post_direction
        self.building = building
        self.premises_type = premises_type
        self.premises_number = premises_number
        self.sub_premises_type = sub_premises_type
        self.sub_premises_number = sub_premises_number
        self.post_box = post_box
        self.latitude = latitude
        self.longitude = longitude
        self.delivery_indicator = delivery_indicator
        self.melissa_address_key = melissa_address_key
        self.melissa_address_key_base = melissa_address_key_base
        self.post_office_location = post_office_location
        self.sub_premise_level = sub_premise_level
        self.sub_premise_level_type = sub_premise_level_type
        self.sub_premise_level_number = sub_premise_level_number
        self.sub_building = sub_building
        self.sub_building_type = sub_building_type
        self.sub_building_number = sub_building_number
        self.utc = utc
        self.dst = dst
        self.delivery_point_suffix = delivery_point_suffix
        self.census_key = census_key
        self.extras = extras

    @classmethod
    def from_dict(cls, data: dict):
        cls.data = data
        return cls(
            record_id = data.get("RecordID", ""),
            results = data.get("Results", ""),
            formatted_address = data.get("FormattedAddress", ""),
            organization = data.get("Organization", ""),
            address_line_1 = data.get("AddressLine1", ""),
            address_line_2 = data.get("AddressLine2", ""),
            address_line_3 = data.get("AddressLine3", ""),
            address_line_4 = data.get("AddressLine4", ""),
            address_line_5 = data.get("AddressLine5", ""),
            address_line_6 = data.get("AddressLine6", ""),
            address_line_7 = data.get("AddressLine7", ""),
            address_line_8 = data.get("AddressLine8", ""),
            sub_premises = data.get("SubPremises", ""),
            double_dependent_locality = data.get("DoubleDependentLocality", ""),
            dependent_locality = data.get("DependentLocality", ""),
            locality = data.get("Locality", ""),
            sub_administrative_area = data.get("SubAdministrativeArea", ""),
            administrative_area = data.get("AdministrativeArea", ""),
            postal_code = data.get("PostalCode", ""),
            postal_code_type = data.get("PostalCodeType", ""),
            address_type = data.get("AddressType", ""),
            address_key = data.get("AddressKey", ""),
            sub_national_area = data.get("SubNationalArea", ""),
            country_name = data.get("CountryName", ""),
            country_iso3166_1_alpha2 = data.get("CountryISO3166_1_Alpha2", ""),
            country_iso3166_1_alpha3 = data.get("CountryISO3166_1_Alpha3", ""),
            country_iso3166_1_numeric = data.get("CountryISO3166_1_Numeric", ""),
            country_subdivision_code = data.get("CountrySubdivisionCode", ""),
            thoroughfare = data.get("Thoroughfare", ""),
            thoroughfare_pre_direction = data.get("ThoroughfarePreDirection", ""),
            thoroughfare_leading_type = data.get("ThoroughfareLeadingType", ""),
            thoroughfare_name = data.get("ThoroughfareName", ""),
            thoroughfare_trailing_type = data.get("ThoroughfareTrailingType", ""),
            thoroughfare_post_direction = data.get("ThoroughfarePostDirection", ""),
            dependent_thoroughfare = data.get("DependentThoroughfare", ""),
            dependent_thoroughfare_pre_direction = data.get("DependentThoroughfarePreDirection", ""),
            dependent_thoroughfare_leading_type = data.get("DependentThoroughfareLeadingType", ""),
            dependent_thoroughfare_name = data.get("DependentThoroughfareName", ""),
            dependent_thoroughfare_trailing_type = data.get("DependentThoroughfareTrailingType", ""),
            dependent_thoroughfare_post_direction = data.get("DependentThoroughfarePostDirection", ""),
            building = data.get("Building", ""),
            premises_type = data.get("PremisesType", ""),
            premises_number = data.get("PremisesNumber", ""),
            sub_premises_type = data.get("SubPremisesType", ""),
            sub_premises_number = data.get("SubPremisesNumber", ""),
            post_box = data.get("PostBox", ""),
            latitude = data.get("Latitude", ""),
            longitude = data.get("Longitude", ""),
            delivery_indicator = data.get("DeliveryIndicator", ""),
            melissa_address_key = data.get("MelissaAddressKey", ""),
            melissa_address_key_base = data.get("MelissaAddressKeyBase", ""),
            post_office_location = data.get("PostOfficeLocation", ""),
            sub_premise_level = data.get("SubPremiseLevel", ""),
            sub_premise_level_type = data.get("SubPremiseLevelType", ""),
            sub_premise_level_number = data.get("SubPremiseLevelNumber", ""),
            sub_building = data.get("SubBuilding", ""),
            sub_building_type = data.get("SubBuildingType", ""),
            sub_building_number = data.get("SubBuildingNumber", ""),
            utc = data.get("UTC", ""),
            dst = data.get("DST", ""),
            delivery_point_suffix = data.get("DeliveryPointSuffix", ""),
            census_key = data.get("CensusKey", ""),
            extras = GlobalAddressVerificationExtras.from_dict(data.get("Extras", None))
        )
    
    # Setters
    def set_record_id(self, record_id):
        self.record_id = record_id

    def set_results(self, results):
        self.results = results

    def set_formatted_address(self, formatted_address):
        self.formatted_address = formatted_address

    def set_organization(self, organization):
        self.organization = organization

    def set_address_line_1(self, address_line_1):
        self.address_line_1 = address_line_1

    def set_address_line_2(self, address_line_2):
        self.address_line_2 = address_line_2

    def set_address_line_3(self, address_line_3):
        self.address_line_3 = address_line_3

    def set_address_line_4(self, address_line_4):
        self.address_line_4 = address_line_4

    def set_address_line_5(self, address_line_5):
        self.address_line_5 = address_line_5

    def set_address_line_6(self, address_line_6):
        self.address_line_6 = address_line_6

    def set_address_line_7(self, address_line_7):
        self.address_line_7 = address_line_7

    def set_address_line_8(self, address_line_8):
        self.address_line_8 = address_line_8

    def set_sub_premises(self, sub_premises):
        self.sub_premises = sub_premises

    def set_double_dependent_locality(self, double_dependent_locality):
        self.double_dependent_locality = double_dependent_locality

    def set_dependent_locality(self, dependent_locality):
        self.dependent_locality = dependent_locality

    def set_locality(self, locality):
        self.locality = locality

    def set_sub_administrative_area(self, sub_administrative_area):
        self.sub_administrative_area = sub_administrative_area

    def set_administrative_area(self, administrative_area):
        self.administrative_area = administrative_area

    def set_postal_code(self, postal_code):
        self.postal_code = postal_code

    def set_postal_code_type(self, postal_code_type):
        self.postal_code_type = postal_code_type

    def set_address_type(self, address_type):
        self.address_type = address_type

    def set_address_key(self, address_key):
        self.address_key = address_key

    def set_sub_national_area(self, sub_national_area):
        self.sub_national_area = sub_national_area

    def set_country_name(self, country_name):
        self.country_name = country_name

    def set_country_iso3166_1_alpha2(self, country_iso3166_1_alpha2):
        self.country_iso3166_1_alpha2 = country_iso3166_1_alpha2

    def set_country_iso3166_1_alpha3(self, country_iso3166_1_alpha3):
        self.country_iso3166_1_alpha3 = country_iso3166_1_alpha3

    def set_country_iso3166_1_numeric(self, country_iso3166_1_numeric):
        self.country_iso3166_1_numeric = country_iso3166_1_numeric

    def set_country_subdivision_code(self, country_subdivision_code):
        self.country_subdivision_code = country_subdivision_code

    def set_thoroughfare(self, thoroughfare):
        self.thoroughfare = thoroughfare

    def set_thoroughfare_pre_direction(self, thoroughfare_pre_direction):
        self.thoroughfare_pre_direction = thoroughfare_pre_direction

    def set_thoroughfare_leading_type(self, thoroughfare_leading_type):
        self.thoroughfare_leading_type = thoroughfare_leading_type

    def set_thoroughfare_name(self, thoroughfare_name):
        self.thoroughfare_name = thoroughfare_name

    def set_thoroughfare_trailing_type(self, thoroughfare_trailing_type):
        self.thoroughfare_trailing_type = thoroughfare_trailing_type

    def set_thoroughfare_post_direction(self, thoroughfare_post_direction):
        self.thoroughfare_post_direction = thoroughfare_post_direction

    def set_dependent_thoroughfare(self, dependent_thoroughfare):
        self.dependent_thoroughfare = dependent_thoroughfare

    def set_dependent_thoroughfare_pre_direction(self, dependent_thoroughfare_pre_direction):
        self.dependent_thoroughfare_pre_direction = dependent_thoroughfare_pre_direction

    def set_dependent_thoroughfare_leading_type(self, dependent_thoroughfare_leading_type):
        self.dependent_thoroughfare_leading_type = dependent_thoroughfare_leading_type

    def set_dependent_thoroughfare_name(self, dependent_thoroughfare_name):
        self.dependent_thoroughfare_name = dependent_thoroughfare_name

    def set_dependent_thoroughfare_trailing_type(self, dependent_thoroughfare_trailing_type):
        self.dependent_thoroughfare_trailing_type = dependent_thoroughfare_trailing_type

    def set_dependent_thoroughfare_post_direction(self, dependent_thoroughfare_post_direction):
        self.dependent_thoroughfare_post_direction = dependent_thoroughfare_post_direction

    def set_building(self, building):
        self.building = building

    def set_premises_type(self, premises_type):
        self.premises_type = premises_type

    def set_premises_number(self, premises_number):
        self.premises_number = premises_number

    def set_sub_premises_type(self, sub_premises_type):
        self.sub_premises_type = sub_premises_type

    def set_sub_premises_number(self, sub_premises_number):
        self.sub_premises_number = sub_premises_number

    def set_post_box(self, post_box):
        self.post_box = post_box

    def set_latitude(self, latitude):
        self.latitude = latitude

    def set_longitude(self, longitude):
        self.longitude = longitude

    def set_delivery_indicator(self, delivery_indicator):
        self.delivery_indicator = delivery_indicator

    def set_melissa_address_key(self, melissa_address_key):
        self.melissa_address_key = melissa_address_key

    def set_melissa_address_key_base(self, melissa_address_key_base):
        self.melissa_address_key_base = melissa_address_key_base

    def set_post_office_location(self, post_office_location):
        self.post_office_location = post_office_location

    def set_sub_premise_level(self, sub_premise_level):
        self.sub_premise_level = sub_premise_level

    def set_sub_premise_level_type(self, sub_premise_level_type):
        self.sub_premise_level_type = sub_premise_level_type

    def set_sub_premise_level_number(self, sub_premise_level_number):
        self.sub_premise_level_number = sub_premise_level_number

    def set_sub_building(self, sub_building):
        self.sub_building = sub_building

    def set_sub_building_type(self, sub_building_type):
        self.sub_building_type = sub_building_type

    def set_sub_building_number(self, sub_building_number):
        self.sub_building_number = sub_building_number

    def set_utc(self, utc):
        self.utc = utc

    def set_dst(self, dst):
        self.dst = dst

    def set_delivery_point_suffix(self, delivery_point):
        self.delivery_point_suffix = delivery_point

    def set_census_key(self, census_key):
        self.census_key = census_key

    # Getters

    def get_record_id(self):
        return self.record_id or ""

    def get_results(self):
        return self.results or ""

    def get_formatted_address(self):
        return self.formatted_address or ""

    def get_organization(self):
        return self.organization or ""

    def get_address_line_1(self):
        return self.address_line_1 or ""

    def get_address_line_2(self):
        return self.address_line_2 or ""

    def get_address_line_3(self):
        return self.address_line_3 or ""

    def get_address_line_4(self):
        return self.address_line_4 or ""

    def get_address_line_5(self):
        return self.address_line_5 or ""

    def get_address_line_6(self):
        return self.address_line_6 or ""

    def get_address_line_7(self):
        return self.address_line_7 or ""

    def get_address_line_8(self):
        return self.address_line_8 or ""

    def get_sub_premises(self):
        return self.sub_premises or ""

    def get_double_dependent_locality(self):
        return self.double_dependent_locality or ""

    def get_dependent_locality(self):
        return self.dependent_locality or ""

    def get_locality(self):
        return self.locality or ""

    def get_sub_administrative_area(self):
        return self.sub_administrative_area or ""

    def get_administrative_area(self):
        return self.administrative_area or ""

    def get_postal_code(self):
        return self.postal_code or ""

    def get_postal_code_type(self):
        return self.postal_code_type or ""

    def get_address_type(self):
        return self.address_type or ""

    def get_address_key(self):
        return self.address_key or ""

    def get_sub_national_area(self):
        return self.sub_national_area or ""

    def get_country_name(self):
        return self.country_name or ""

    def get_country_iso3166_1_alpha2(self):
        return self.country_iso3166_1_alpha2 or ""

    def get_country_iso3166_1_alpha3(self):
        return self.country_iso3166_1_alpha3 or ""

    def get_country_iso3166_1_numeric(self):
        return self.country_iso3166_1_numeric or ""

    def get_country_subdivision_code(self):
        return self.country_subdivision_code or ""

    def get_thoroughfare(self):
        return self.thoroughfare or ""

    def get_thoroughfare_pre_direction(self):
        return self.thoroughfare_pre_direction or ""

    def get_thoroughfare_leading_type(self):
        return self.thoroughfare_leading_type or ""

    def get_thoroughfare_name(self):
        return self.thoroughfare_name or ""

    def get_thoroughfare_trailing_type(self):
        return self.thoroughfare_trailing_type or ""

    def get_thoroughfare_post_direction(self):
        return self.thoroughfare_post_direction or ""

    def get_dependent_thoroughfare(self):
        return self.dependent_thoroughfare or ""

    def get_dependent_thoroughfare_pre_direction(self):
        return self.dependent_thoroughfare_pre_direction or ""

    def get_dependent_thoroughfare_leading_type(self):
        return self.dependent_thoroughfare_leading_type or ""

    def get_dependent_thoroughfare_name(self):
        return self.dependent_thoroughfare_name or ""

    def get_dependent_thoroughfare_trailing_type(self):
        return self.dependent_thoroughfare_trailing_type or ""

    def get_dependent_thoroughfare_post_direction(self):
        return self.dependent_thoroughfare_post_direction or ""

    def get_building(self):
        return self.building or ""

    def get_premises_type(self):
        return self.premises_type or ""

    def get_premises_number(self):
        return self.premises_number or ""

    def get_sub_premises_type(self):
        return self.sub_premises_type or ""

    def get_sub_premises_number(self):
        return self.sub_premises_number or ""

    def get_post_box(self):
        return self.post_box or ""

    def get_latitude(self):
        return self.latitude or ""

    def get_longitude(self):
        return self.longitude or ""

    def get_delivery_indicator(self):
        return self.delivery_indicator or ""

    def get_melissa_address_key(self):
        return self.melissa_address_key or ""

    def get_melissa_address_key_base(self):
        return self.melissa_address_key_base or ""

    def get_post_office_location(self):
        return self.post_office_location or ""

    def get_sub_premise_level(self):
        return self.sub_premise_level or ""

    def get_sub_premise_level_type(self):
        return self.sub_premise_level_type or ""

    def get_sub_premise_level_number(self):
        return self.sub_premise_level_number or ""

    def get_sub_building(self):
        return self.sub_building or ""

    def get_sub_building_type(self):
        return self.sub_building_type or ""

    def get_sub_building_number(self):
        return self.sub_building_number or ""

    def get_utc(self):
        return self.utc or ""

    def get_dst(self):
        return self.dst or ""

    def get_delivery_point_suffix(self):
        return self.delivery_point_suffix or ""

    def get_census_key(self):
        return self.census_key or ""
    

class GlobalAddressVerificationExtras(ResponseBase):
    def __init__(self, daylight_savings_timezone="", daylight_savings_utc="", local_date_time="", 
             standard_timezone="", uprn="", carrier_route="", cbsa_code="", cbsa_division_code="", 
             cbsa_division_level="", cbsa_division_title="", cbsa_level="", cbsa_title="", 
             census_block="", census_tract="", congressional_district="", county_fips="", 
             county_name="", county_subdivision_code="", county_subdivision_name="", 
             delivery_point_check_digit="", delivery_point_code="", elementary_school_district_code="", 
             elementary_school_district_name="", place_code="", place_name="", 
             secondary_school_district_code="", secondary_school_district_name="", 
             state_district_lower="", state_district_upper="", unified_school_district_code="", 
             unified_school_district_name=""):
        self.daylight_savings_timezone = daylight_savings_timezone
        self.daylight_savings_utc = daylight_savings_utc
        self.local_date_time = local_date_time
        self.standard_timezone = standard_timezone
        self.uprn = uprn
        self.carrier_route = carrier_route
        self.cbsa_code = cbsa_code
        self.cbsa_division_code = cbsa_division_code
        self.cbsa_division_level = cbsa_division_level
        self.cbsa_division_title = cbsa_division_title
        self.cbsa_level = cbsa_level
        self.cbsa_title = cbsa_title
        self.census_block = census_block
        self.census_tract = census_tract
        self.congressional_district = congressional_district
        self.county_fips = county_fips
        self.county_name = county_name
        self.county_subdivision_code = county_subdivision_code
        self.county_subdivision_name = county_subdivision_name
        self.delivery_point_check_digit = delivery_point_check_digit
        self.delivery_point_code = delivery_point_code
        self.elementary_school_district_code = elementary_school_district_code
        self.elementary_school_district_name = elementary_school_district_name
        self.place_code = place_code
        self.place_name = place_name
        self.secondary_school_district_code = secondary_school_district_code
        self.secondary_school_district_name = secondary_school_district_name
        self.state_district_lower = state_district_lower
        self.state_district_upper = state_district_upper
        self.unified_school_district_code = unified_school_district_code
        self.unified_school_district_name = unified_school_district_name


    @classmethod
    def from_dict(cls, data: dict):
        cls.data = data
        return cls(
            daylight_savings_timezone=data.get("DaylightSavingsTimezone", ""),
            daylight_savings_utc=data.get("DaylightSavingsUTC", ""),
            local_date_time=data.get("LocalDateTime", ""),
            standard_timezone=data.get("StandardTimezone", ""),
            uprn=data.get("UPRN", ""),
            carrier_route=data.get("CarrierRoute", ""),
            cbsa_code=data.get("CBSACode", ""),
            cbsa_division_code=data.get("CBSADivisionCode", ""),
            cbsa_division_level=data.get("CBSADivisionLevel", ""),
            cbsa_division_title=data.get("CBSADivisionTitle", ""),
            cbsa_level=data.get("CBSALevel", ""),
            cbsa_title=data.get("CBSATitle", ""),
            census_block=data.get("CensusBlock", ""),
            census_tract=data.get("CensusTract", ""),
            congressional_district=data.get("CongressionalDistrict", ""),
            county_fips=data.get("CountyFIPS", ""),
            county_name=data.get("CountyName", ""),
            county_subdivision_code=data.get("CountySubdivisionCode", ""),
            county_subdivision_name=data.get("CountySubdivisionName", ""),
            delivery_point_check_digit=data.get("DeliveryPointCheckDigit", ""),
            delivery_point_code=data.get("DeliveryPointCode", ""),
            elementary_school_district_code=data.get("ElementarySchoolDistrictCode", ""),
            elementary_school_district_name=data.get("ElementarySchoolDistrictName", ""),
            place_code=data.get("PlaceCode", ""),
            place_name=data.get("PlaceName", ""),
            secondary_school_district_code=data.get("SecondarySchoolDistrictCode", ""),
            secondary_school_district_name=data.get("SecondarySchoolDistrictName", ""),
            state_district_lower=data.get("StateDistrictLower", ""),
            state_district_upper=data.get("StateDistrictUpper", ""),
            unified_school_district_code=data.get("UnifiedSchoolDistrictCode", ""),
            unified_school_district_name=data.get("UnifiedSchoolDistrictName", "")
        )
    
    # Setters
    def set_daylight_savings_timezone(self, daylight_savings_timezone):
        self.daylight_savings_timezone = daylight_savings_timezone

    def set_daylight_savings_utc(self, daylight_savings_utc):
        self.daylight_savings_utc = daylight_savings_utc

    def set_local_date_time(self, local_date_time):
        self.local_date_time = local_date_time

    def set_standard_timezone(self, standard_timezone):
        self.standard_timezone = standard_timezone

    def set_uprn(self, uprn):
        self.uprn = uprn

    def set_carrier_route(self, carrier_route):
        self.carrier_route = carrier_route

    def set_cbsa_code(self, cbsa_code):
        self.cbsa_code = cbsa_code

    def set_cbsa_division_code(self, cbsa_division_code):
        self.cbsa_division_code = cbsa_division_code

    def set_cbsa_division_level(self, cbsa_division_level):
        self.cbsa_division_level = cbsa_division_level

    def set_cbsa_division_title(self, cbsa_division_title):
        self.cbsa_division_title = cbsa_division_title

    def set_cbsa_level(self, cbsa_level):
        self.cbsa_level = cbsa_level

    def set_cbsa_title(self, cbsa_title):
        self.cbsa_title = cbsa_title

    def set_census_block(self, census_block):
        self.census_block = census_block

    def set_census_tract(self, census_tract):
        self.census_tract = census_tract

    def set_congressional_district(self, congressional_district):
        self.congressional_district = congressional_district

    def set_county_fips(self, county_fips):
        self.county_fips = county_fips

    def set_county_name(self, county_name):
        self.county_name = county_name

    def set_county_subdivision_code(self, county_subdivision_code):
        self.county_subdivision_code = county_subdivision_code

    def set_county_subdivision_name(self, county_subdivision_name):
        self.county_subdivision_name = county_subdivision_name

    def set_delivery_point_check_digit(self, delivery_point_check_digit):
        self.delivery_point_check_digit = delivery_point_check_digit

    def set_delivery_point_code(self, delivery_point_code):
        self.delivery_point_code = delivery_point_code

    def set_elementary_school_district_code(self, elementary_school_district_code):
        self.elementary_school_district_code = elementary_school_district_code

    def set_elementary_school_district_name(self, elementary_school_district_name):
        self.elementary_school_district_name = elementary_school_district_name

    def set_place_code(self, place_code):
        self.place_code = place_code

    def set_place_name(self, place_name):
        self.place_name = place_name

    def set_secondary_school_district_code(self, secondary_school_district_code):
        self.secondary_school_district_code = secondary_school_district_code

    def set_secondary_school_district_name(self, secondary_school_district_name):
        self.secondary_school_district_name = secondary_school_district_name

    def set_state_district_lower(self, state_district_lower):
        self.state_district_lower = state_district_lower

    def set_state_district_upper(self, state_district_upper):
        self.state_district_upper = state_district_upper

    def set_unified_school_district_code(self, unified_school_district_code):
        self.unified_school_district_code = unified_school_district_code

    def set_unified_school_district_name(self, unified_school_district_name):
        self.unified_school_district_name = unified_school_district_name


    # Getters
    def get_daylight_savings_timezone(self):
        return self.daylight_savings_timezone or ""

    def get_daylight_savings_utc(self):
        return self.daylight_savings_utc or ""

    def get_local_date_time(self):
        return self.local_date_time or ""

    def get_standard_timezone(self):
        return self.standard_timezone or ""

    def get_uprn(self):
        return self.uprn or ""

    def get_carrier_route(self):
        return self.carrier_route or ""

    def get_cbsa_code(self):
        return self.cbsa_code or ""

    def get_cbsa_division_code(self):
        return self.cbsa_division_code or ""

    def get_cbsa_division_level(self):
        return self.cbsa_division_level or ""

    def get_cbsa_division_title(self):
        return self.cbsa_division_title or ""

    def get_cbsa_level(self):
        return self.cbsa_level or ""

    def get_cbsa_title(self):
        return self.cbsa_title or ""

    def get_census_block(self):
        return self.census_block or ""

    def get_census_tract(self):
        return self.census_tract or ""

    def get_congressional_district(self):
        return self.congressional_district or ""

    def get_county_fips(self):
        return self.county_fips or ""

    def get_county_name(self):
        return self.county_name or ""

    def get_county_subdivision_code(self):
        return self.county_subdivision_code or ""

    def get_county_subdivision_name(self):
        return self.county_subdivision_name or ""

    def get_delivery_point_check_digit(self):
        return self.delivery_point_check_digit or ""

    def get_delivery_point_code(self):
        return self.delivery_point_code or ""

    def get_elementary_school_district_code(self):
        return self.elementary_school_district_code or ""

    def get_elementary_school_district_name(self):
        return self.elementary_school_district_name or ""

    def get_place_code(self):
        return self.place_code or ""

    def get_place_name(self):
        return self.place_name or ""

    def get_secondary_school_district_code(self):
        return self.secondary_school_district_code or ""

    def get_secondary_school_district_name(self):
        return self.secondary_school_district_name or ""

    def get_state_district_lower(self):
        return self.state_district_lower or ""

    def get_state_district_upper(self):
        return self.state_district_upper or ""

    def get_unified_school_district_code(self):
        return self.unified_school_district_code or ""

    def get_unified_school_district_name(self):
        return self.unified_school_district_name or ""









import json
import os
from .CloudApiBase import CloudApiBase
from ..PostReqestBase import GlobalPhonePostRequest


class GlobalPhone(CloudApiBase):

    def __init__(self, license=None):
        super().__init__("https://globalphone.melissadata.net", "/v4/WEB/GlobalPhone/doGlobalPhone")
        self.license = license or os.environ.get("MD_LICENSE")
        self.parameters["format"] = "json"
        self.records = []
        self.post_body = None

        self.parameter_mappings = {
            "phone": "phone",
            "country": "ctry",
            "country_origin": "ctryOrg",
            "transmission_reference": "t",
            "opt": "opt"
        }

        # Initialize private backing fields
        for attr in self.parameter_mappings.keys():
            setattr(self, f"_{attr}", None)

    def _set_parameter(self, attr_name, value):
        """
        Helper method to set a private attribute and update the parameters dictionary.
        """
        setattr(self, f"_{attr_name}", value)
        self.parameters[self.parameter_mappings[attr_name]] = value

    # Properties
    @property
    def phone(self):
        return self._phone

    @phone.setter
    def phone(self, value):
        self._set_parameter("phone", value)

    @property
    def country(self):
        return self._country

    @country.setter
    def country(self, value):
        self._set_parameter("country", value)

    @property
    def country_origin(self):
        return self._country_origin

    @country_origin.setter
    def country_origin(self, value):
        self._set_parameter("country_origin", value)

    @property
    def transmission_reference(self):
        return self._transmission_reference

    @transmission_reference.setter
    def transmission_reference(self, value):
        self._set_parameter("transmission_reference", value)

    @property
    def opt(self):
        return self._opt

    @opt.setter
    def opt(self, value):
        self._set_parameter("opt", value)

    # Setters
    def set_phone(self, value):
        self._set_parameter("phone", value)

    def set_country(self, value):
        self._set_parameter("country", value)

    def set_country_origin(self, value):
        self._set_parameter("country_origin", value)

    def set_transmission_reference(self, value):
        self._set_parameter("transmission_reference", value)

    def set_opt(self, value):
        self._set_parameter("opt", value)

    def set_post_body(self, post_body):
        self.post_body = post_body

    def set_value(self, parameter, value):
        parameter = parameter.strip() if parameter else None

        if parameter in self.parameter_mappings:
            parameter_key = self.parameter_mappings[parameter]

            property_name = next((key for key, val in self.parameter_mappings.items()
                                  if val.lower() == parameter_key.lower()), None)

            if property_name:
                if hasattr(self, property_name):
                    setattr(self, property_name, value)
        else:
            # If not in derived class, go to base class implementation
            super().set_value(parameter, value)

    # Getters
    def get_phone(self):
        return self.parameters.get("phone") or ""

    def get_country(self):
        return self.parameters.get("ctry") or ""

    def get_country_origin(self):
        return self.parameters.get("ctryOrg") or ""

    def get_transmission_reference(self):
        return self.parameters.get("t") or ""

    def get_opt(self):
        return self.parameters.get("opt") or ""

    def get_post_body(self):
        return self.post_body or ""

    # Class Methods
    def add_record(self, request):
        self.records.append(request)

    def clear_records(self):
        self.records.clear()

    def post(self, response_type=None):
        """
        This synchronous function makes the batch request using the post body.

        Returns:
            A string or deserialized json response object.
        """

        if self.records and len(self.records) > 0:
            post_request = GlobalPhonePostRequest(
                transmission_reference=self.get_transmission_reference(),
                records=self.records,
                customer_id=self.get_license()
            )
            response = self.send_post_request(post_request)
        else:
            response = self.send_post_request(self.post_body)
        return self.handle_response(response, response_type)

    def get_api_version(self):
        """
        Makes a synchronous getversion request and parses the response for the API version.

        Returns:
            The Cloud API version.
        """
        json_response = self.send_version_request()
        response_objectect = json.loads(json_response)
        version = response_objectect["Version"]
        return version



import json
import os
from .CloudApiBase import CloudApiBase
from ..PostReqestBase import BusinessCoderPostRequest

class BusinessCoder(CloudApiBase):
    def __init__(self, license=None):

        super().__init__("https://businesscoder.melissadata.net", "/WEB/BusinessCoder/doBusinessCoderUS")
        self.license = license or os.environ.get("MD_LICENSE")
        self.parameters["format"] = "json"
        self.records = []
        self.post_body = None

        self.parameter_mappings = {
            "record_id": "rec",
            "company": "comp",
            "phone": "phone",
            "address_line_1": "a1",
            "address_line_2": "a2",
            "suite": "suite",
            "city": "city",
            "state": "state",
            "postal": "postal",
            "country": "ctry",
            "mak": "mak",
            "stock_ticker": "stock",
            "web_address": "web",
            "mek": "mek",
            "free_form": "freeform",
            "transmission_reference": "t",
            "opt": "opt",
            "cols": "cols",
        }

        # Initialize private backing fields
        for attr in self.parameter_mappings.keys():
            setattr(self, f"_{attr}", None)

        
    def _set_parameter(self, attr_name, value):
        """
        Helper method to set a private attribute and update the parameters dictionary.
        """
        setattr(self, f"_{attr_name}", value)
        self.parameters[self.parameter_mappings[attr_name]] = value

    # Properties
    @property
    def record_id(self):
        return self._record_id

    @record_id.setter
    def record_id(self, value):
        self._set_parameter("record_id", value)

    @property
    def company(self):
        return self._company

    @company.setter
    def company(self, value):
        self._set_parameter("company", value)

    @property
    def phone(self):
        return self._phone

    @phone.setter
    def phone(self, value):
        self._set_parameter("phone", value)

    @property
    def address_line_1(self):
        return self._address_line_1

    @address_line_1.setter
    def address_line_1(self, value):
        self._set_parameter("address_line_1", value)

    @property
    def address_line_2(self):
        return self._address_line_2

    @address_line_2.setter
    def address_line_2(self, value):
        self._set_parameter("address_line_2", value)

    @property
    def suite(self):
        return self._suite

    @suite.setter
    def suite(self, value):
        self._set_parameter("suite", value)

    @property
    def city(self):
        return self._city

    @city.setter
    def city(self, value):
        self._set_parameter("city", value)

    @property
    def state(self):
        return self._state

    @state.setter
    def state(self, value):
        self._set_parameter("state", value)

    @property
    def postal(self):
        return self._postal

    @postal.setter
    def postal(self, value):
        self._set_parameter("postal", value)

    @property
    def country(self):
        return self._country

    @country.setter
    def country(self, value):
        self._set_parameter("country", value)

    @property
    def mak(self):
        return self._mak

    @mak.setter
    def mak(self, value):
        self._set_parameter("mak", value)

    @property
    def stock_ticker(self):
        return self._stock_ticker

    @stock_ticker.setter
    def stock_ticker(self, value):
        self._set_parameter("stock_ticker", value)

    @property
    def web_address(self):
        return self._web_address

    @web_address.setter
    def web_address(self, value):
        self._set_parameter("web_address", value)

    @property
    def mek(self):
        return self._mek

    @mek.setter
    def mek(self, value):
        self._set_parameter("mek", value)

    @property
    def free_form(self):
        return self._free_form

    @free_form.setter
    def free_form(self, value):
        self._set_parameter("free_form", value)

    @property
    def transmission_reference(self):
        return self._transmission_reference

    @transmission_reference.setter
    def transmission_reference(self, value):
        self._set_parameter("transmission_reference", value)

    @property
    def opt(self):
        return self._opt

    @opt.setter
    def opt(self, value):
        self._set_parameter("opt", value)

    @property
    def cols(self):
        return self._cols

    @cols.setter
    def cols(self, value):
        self._set_parameter("cols", value)


    # Setters
    def set_record_id(self, record_id):
        self._set_parameter("record_id", record_id)

    def set_company(self, company):
        self._set_parameter("company", company)

    def set_phone(self, phone):
        self._set_parameter("phone", phone)

    def set_address_line_1(self, address_line_1):
        self._set_parameter("address_line_1", address_line_1)

    def set_address_line_2(self, address_line_2):
        self._set_parameter("address_line_2", address_line_2)

    def set_suite(self, suite):
        self._set_parameter("suite", suite)

    def set_city(self, city):
        self._set_parameter("city", city)

    def set_state(self, state):
        self._set_parameter("state", state)

    def set_postal(self, postal):
        self._set_parameter("postal", postal)

    def set_country(self, country):
        self._set_parameter("country", country)

    def set_mak(self, mak):
        self._set_parameter("mak", mak)

    def set_stock_ticker(self, ticker):
        self._set_parameter("stock_ticker", ticker)

    def set_web_address(self, url):
        self._set_parameter("web_address", url)

    def set_mek(self, mek):
        self._set_parameter("mek", mek)

    def set_free_form(self, ff):
        self._set_parameter("free_form", ff)

    def set_transmission_reference(self, transmission_reference):
        self._set_parameter("transmission_reference", transmission_reference)

    def set_opt(self, opt):
        self._set_parameter("opt", opt)

    def set_cols(self, cols):
        self._set_parameter("cols", cols)

    def set_post_body(self, post_body):
        self.post_body = post_body

    def set_value(self, parameter, value):
        parameter = parameter.strip() if parameter else None
        
        if parameter in self.parameter_mappings:
            parameter_key = self.parameter_mappings[parameter]
            
            property_name = next((key for key, val in self.parameter_mappings.items() 
                                  if val.lower() == parameter_key.lower()), None)
            
            if property_name:
                if hasattr(self, property_name):
                    setattr(self, property_name, value)
        else:
            # If not in derived class, go to base class implementation
            super().set_value(parameter, value)

    # Getters
    def get_record_id(self):
        return self.parameters.get("rec", "")

    def get_company(self):
        return self.parameters.get("comp", "")

    def get_phone(self):
        return self.parameters.get("phone", "")

    def get_address_line_1(self):
        return self.parameters.get("a1", "")

    def get_address_line_2(self):
        return self.parameters.get("a2", "")

    def get_suite(self):
        return self.parameters.get("suite", "")

    def get_city(self):
        return self.parameters.get("city", "")

    def get_state(self):
        return self.parameters.get("state", "")

    def get_postal(self):
        return self.parameters.get("postal", "")

    def get_country(self):
        return self.parameters.get("ctry", "")

    def get_mak(self):
        return self.parameters.get("mak", "")

    def get_stock_ticker(self):
        return self.parameters.get("stock", "")

    def get_web_address(self):
        return self.parameters.get("web", "")

    def get_mek(self):
        return self.parameters.get("mek", "")

    def get_free_form(self):
        return self.parameters.get("freeform", "")

    def get_transmission_reference(self):
        return self.parameters.get("t", "")

    def get_opt(self):
        return self.parameters.get("opt", "")

    def get_cols(self):
        return self.parameters.get("cols", "")

    def get_post_body(self):
        return self.post_body

    # Class Methods
    def add_record(self, record):
        self.records.append(record)

    def clear_records(self):
        self.records.clear()

    def post(self, response_type):
        """
        This synchronous function makes the batch request using the post body.

        Returns:
            A string or deserialized json response object.
        """
        if self.records and len(self.records) > 0:
            post_request = BusinessCoderPostRequest(
                cols=self.get_cols(),
                id=self.get_license(),
                t=self.get_transmission_reference(),
                opt=self.get_opt(),
                records=self.records
            )
            response = self.send_post_request(post_request)
        else:
            response = self.send_post_request(self.post_body)

        return self.handle_response(response, response_type)

    
    def get_api_version(self):
        """
        Makes a synchronous getversion request and parses the response for the API version.

        Returns:
            The Cloud API version.
        """
        json_response = self.send_version_request()
        response_objectect = json.loads(json_response)
        business_coder_version = response_objectect["businessCoderVersion"]
        return business_coder_version

    


import json
import os
from .CloudApiBase import CloudApiBase

class GlobalExpressEntry(CloudApiBase):

    def __init__(self, license):
        super().__init__("https://expressentry.melissadata.net", "/web/ExpressAddress")
        self.license = license or os.environ.get("MD_LICENSE")
        self.parameters["format"] = "json"

        self.parameter_mappings = {
            "address_line_1": "line1",
            "city": "city",
            "state": "state",
            "postal": "postalcode",
            "free_form": "ff",
            "thoroughfare": "thoroughfare",
            "locality": "locality",
            "administrative_area": "administrativearea",
            "country": "country",
            "max_records": "maxrecords",
            "opt": "opt",
            "cols": "cols",
            "native_char_set": "nativecharset"
        }

        # Initialize private backing fields
        for attr in self.parameter_mappings.keys():
            setattr(self, f"_{attr}", None)

    def _set_parameter(self, attr_name, value):
        """
        Helper method to set a private attribute and update the parameters dictionary.
        """
        setattr(self, f"_{attr_name}", value)
        self.parameters[self.parameter_mappings[attr_name]] = value

    # Properties

    @property
    def address_line_1(self):
        return self._address_line_1

    @address_line_1.setter
    def address_line_1(self, value):
        self._set_parameter("address_line_1", value)

    @property
    def city(self):
        return self._city

    @city.setter
    def city(self, value):
        self._set_parameter("city", value)

    @property
    def state(self):
        return self._state

    @state.setter
    def state(self, value):
        self._set_parameter("state", value)

    @property
    def postal(self):
        return self._postal

    @postal.setter
    def postal(self, value):
        self._set_parameter("postal", value)

    @property
    def free_form(self):
        return self._free_form

    @free_form.setter
    def free_form(self, value):
        self._set_parameter("free_form", value)

    @property
    def thoroughfare(self):
        return self._thoroughfare

    @thoroughfare.setter
    def thoroughfare(self, value):
        self._set_parameter("thoroughfare", value)

    @property
    def locality(self):
        return self._locality

    @locality.setter
    def locality(self, value):
        self._set_parameter("locality", value)

    @property
    def administrative_area(self):
        return self._administrative_area

    @administrative_area.setter
    def administrative_area(self, value):
        self._set_parameter("administrative_area", value)

    @property
    def country(self):
        return self._country

    @country.setter
    def country(self, value):
        self._set_parameter("country", value)

    @property
    def max_records(self):
        return self._max_records

    @max_records.setter
    def max_records(self, value):
        self._set_parameter("max_records", value)

    @property
    def opt(self):
        return self._opt

    @opt.setter
    def opt(self, value):
        self._set_parameter("opt", value)

    @property
    def cols(self):
        return self._cols

    @cols.setter
    def cols(self, value):
        self._set_parameter("cols", value)

    @property
    def native_char_set(self):
        return self._native_char_set

    @native_char_set.setter
    def native_char_set(self, value):
        self._set_parameter("native_char_set", value)

    # Setters

    def set_address_line_1(self, address_line_1):
        if self.endpoint.lower() == "web/globalexpressaddress":
            self._set_parameter("address_line_1", self.parameters.get("line1", "")) 
        else:
            self._set_parameter("address_line_1", address_line_1)

    def set_city(self, city):
        self._set_parameter("city", city)

    def set_state(self, state):
        self._set_parameter("state", state)

    def set_postal(self, postal):
        self._set_parameter("postal", postal)

    def set_free_form(self, free_form):
        self._set_parameter("free_form", free_form)

    def set_thoroughfare(self, thoroughfare):
        self._set_parameter("thoroughfare", thoroughfare)

    def set_locality(self, locality):
        self._set_parameter("locality", locality)

    def set_administrative_area(self, administrative_area):
        self._set_parameter("administrative_area", administrative_area)

    def set_country(self, country):
        self._set_parameter("country", country)

    def set_max_records(self, max_records):
        self._set_parameter("max_records", max_records)

    def set_opt(self, opt):
        self._set_parameter("opt", opt)

    def set_cols(self, cols):
        self._set_parameter("cols", cols)

    def set_native_char_set(self, native_char_set):
        self._set_parameter("native_char_set", native_char_set)

    def set_value(self, parameter, value):
        parameter = parameter.strip() if parameter else None

        if parameter in self.parameter_mappings:
            parameter_key = self.parameter_mappings[parameter]

            property_name = next((key for key, val in self.parameter_mappings.items()
                                  if val.lower() == parameter_key.lower()), None)

            if property_name:
                if hasattr(self, property_name):
                    setattr(self, property_name, value)
        else:
            # If not in derived class, go to base class implementation
            super().set_value(parameter, value)


    # Getters

    def get_address_line_1(self):
        if "line1" in self.parameters:
            return self.parameters["line1"]
        elif "address1" in self.parameters:
            return self.parameters["address1"]
        else:
            return ""

    def get_city(self):
        return self.parameters.get("city", "")

    def get_state(self):
        return self.parameters.get("state", "")

    def get_postal(self):
        return self.parameters.get("postalcode", "")

    def get_free_form(self):
        return self.parameters.get("ff", "")

    def get_thoroughfare(self):
        return self.parameters.get("thoroughfare", "")

    def get_locality(self):
        return self.parameters.get("locality", "")

    def get_administrative_area(self):
        return self.parameters.get("administrativearea", "")

    def get_country(self):
        return self.parameters.get("country", "")

    def get_max_records(self):
        return self.parameters.get("maxrecords", "")

    def get_opt(self):
        return self.parameters.get("opt", "")

    def get_cols(self):
        return self.parameters.get("cols", "")

    def get_native_char_set(self):
        return self.parameters.get("nativecharset", "")
    
    
    def get_express_address(self, type):
        """
        Synchronous function sends the CloudAPI request using the /web/ExpressAddress endpoint
        """
        self.set_endpoint("/web/ExpressAddress")
        return self.get(type)

    
    def get_express_city_state(self, type):
        """
        Synchronous function sends the CloudAPI request using the /web/ExpressCityState endpoint
        """
        self.set_endpoint("/web/ExpressCityState")
        return self.get(type)

    
    def get_express_postal_code(self, type):
        """
        Synchronous function sends the CloudAPI request using the /web/ExpressPostalCode endpoint
        """
        self.set_endpoint("/web/ExpressPostalCode")
        return self.get(type)

    
    def get_express_street(self, type):
        """
        Synchronous function sends the CloudAPI request using the /web/ExpressStreet endpoint
        """
        self.set_endpoint("/web/ExpressStreet")
        return self.get(type)

    
    def get_express_free_form(self, type):
        """
        Synchronous function sends the CloudAPI request using the /web/ExpressFreeForm endpoint
        """
        self.set_endpoint("/web/ExpressFreeForm")
        return self.get(type)

    
    def get_global_express_address(self, type):
        """
        Synchronous function sends the CloudAPI request using the /web/GlobalExpressAddress endpoint
        """
        self.set_endpoint("/web/GlobalExpressAddress")
        return self.get(type)

    
    def get_global_express_locality_administrative_area(self, type):
        """
        Synchronous function sends the CloudAPI request using the /web/GlobalExpressLocalityAdministrativeArea endpoint
        """
        self.set_endpoint("/web/GlobalExpressLocalityAdministrativeArea")
        return self.get(type)

    
    def get_global_express_country(self, type):
        """
        Synchronous function sends the CloudAPI request using the /web/GlobalExpressCountry endpoint
        """
        self.set_endpoint("/web/GlobalExpressCountry")
        return self.get(type)

    
    def get_global_express_free_form(self, type):
        """
        Synchronous function sends the CloudAPI request using the /web/GlobalExpressFreeForm endpoint
        """
        self.set_endpoint("/web/GlobalExpressFreeForm")
        return self.get(type)

    
    def get_global_express_postal_code(self, type):
        """
        Synchronous function sends the CloudAPI request using the /web/GlobalExpressPostalCode endpoint
        """
        self.set_endpoint("/web/GlobalExpressPostalCode")
        return self.get(type)

    
    def get_global_express_thoroughfare(self, type):
        """
        Synchronous function sends the CloudAPI request using the /web/GlobalExpressThoroughfare endpoint
        """
        self.set_endpoint("/web/GlobalExpressThoroughfare")
        return self.get(type)

    def get_api_version(self):
        """
        Makes a synchronous getversion request and parses the response for the API version.

        Returns:
            The Cloud API version.
        """
        json_response = self.send_version_request()
        response_objectect = json.loads(json_response)
        version = response_objectect.get("BuildNumber", "")
        return version






from .ResponseBase import ResponseBase

class ExpressEntryResponse(ResponseBase):

    def __init__(self, version="", result_code="", error_string="", results=None):
        self.version = version
        self.result_code = result_code
        self.error_string = error_string
        self.results = results if results is not None else []

    @classmethod
    def populate_from_dict(cls, data: dict):
        cls.data=data
        results = [ExpressEntryRecord.from_dict(result) for result in data.get("Results", [])]
        return cls(
            version=data.get("Version", ""),
            result_code=data.get("ResultCode", ""),
            error_string=data.get("ErrorString", ""),
            results=results
        )

    # Setters
    def set_version(self, version):
        self.version = version

    def set_result_code(self, result_code):
        self.result_code = result_code

    def set_error_string(self, error_string):
        self.error_string = error_string

    # Getters
    def get_version(self):
        return self.version or ""

    def get_result_code(self):
        return self.result_code or ""

    def get_error_string(self):
        return self.error_string or ""
    

class ExpressEntryRecord(ResponseBase):

    def __init__(self, address=None):
        self.address=address

    @classmethod
    def from_dict(cls, data: dict):
        cls.data = data
        return cls(
            address = ExpressEntryAddress.from_dict(data.get("Address", None))
        )

class ExpressEntryAddress(ResponseBase):
    def __init__(self, address_line_1="", city="", city_accepted="", city_not_accepted="", state="",
                 postal_code="", country_subdivision_code="", address_key="", suite_name="", suite_count="",
                 suite_list=None, plus_four=None, mak="", base_mak=""):

        self.address_line_1 = address_line_1
        self.city = city
        self.city_accepted = city_accepted
        self.city_not_accepted = city_not_accepted
        self.state = state
        self.postal_code = postal_code
        self.country_subdivision_code = country_subdivision_code
        self.address_key = address_key
        self.suite_name = suite_name
        self.suite_count = suite_count
        self.suite_list = suite_list or []
        self.plus_four = plus_four or []
        self.mak = mak
        self.base_mak = base_mak

    @classmethod
    def from_dict(cls, data):
        cls.data=data
        return cls(
            address_line_1=data.get("AddressLine1", ""),
            city=data.get("City", ""),
            city_accepted=data.get("CityAccepted", ""),
            city_not_accepted=data.get("CityNotAccepted", ""),
            state=data.get("State", ""),
            postal_code=data.get("PostalCode", ""),
            country_subdivision_code=data.get("CountrySubdivisionCode", ""),
            address_key=data.get("AddressKey", ""),
            suite_name=data.get("SuiteName", ""),
            suite_count=data.get("SuiteCount", ""),
            suite_list=data.get("SuiteList", []),
            plus_four=data.get("PlusFour", []),
            mak=data.get("MAK", ""),
            base_mak=data.get("BaseMAK", "")
        )
    
    # Setters
    def set_address_line_1(self, address_line_1):
        self.address_line_1 = address_line_1

    def set_city(self, city):
        self.city = city

    def set_city_accepted(self, city_accepted):
        self.city_accepted = city_accepted

    def set_city_not_accepted(self, city_not_accepted):
        self.city_not_accepted = city_not_accepted

    def set_state(self, state):
        self.state = state

    def set_postal_code(self, postal_code):
        self.postal_code = postal_code

    def set_country_subdivision_code(self, country_subdivision_code):
        self.country_subdivision_code = country_subdivision_code

    def set_address_key(self, address_key):
        self.address_key = address_key

    def set_suite_name(self, suite_name):
        self.suite_name = suite_name

    def set_suite_count(self, suite_count):
        self.suite_count = suite_count

    def set_mak(self, mak):
        self.mak = mak

    def set_base_mak(self, base_mak):
        self.base_mak = base_mak

    # Getters
    def get_address_line_1(self):
        return self.address_line_1 or ""

    def get_city(self):
        return self.city or ""

    def get_city_accepted(self):
        return self.city_accepted or ""

    def get_city_not_accepted(self):
        return self.city_not_accepted or ""

    def get_state(self):
        return self.state or ""

    def get_postal_code(self):
        return self.postal_code or ""

    def get_country_subdivision_code(self):
        return self.country_subdivision_code or ""

    def get_address_key(self):
        return self.address_key or ""

    def get_suite_name(self):
        return self.suite_name or ""

    def get_suite_count(self):
        return self.suite_count or ""

    def get_mak(self):
        return self.mak or ""

    def get_base_mak(self):
        return self.base_mak or ""




from .ResponseBase import ResponseBase


class PeopleBusinessSearchResponse(ResponseBase):
    def __init__(self, version="", transmission_reference="", total_records="", result_code="", results=None):
        self.version = version
        self.transmission_reference = transmission_reference
        self.result_code = result_code
        self.total_records = total_records
        self.results = results if results is not None else []

    @classmethod
    def populate_from_dict(cls, data: dict):
        cls.data = data
        results = [PeopleBusinessSearchResult.from_dict(result) for result in data.get("Results", [])]
        return cls(
            version=data.get("Version", ""),
            transmission_reference=data.get("TransmissionReference", ""),
            result_code = data.get("ResultCode", ""),
            total_records = data.get("TotalRecords", ""),
            results=results
        )
    
    # Setters
    def set_version(self, version):
        self.version = version

    def set_transmission_reference(self, transmission_reference):
        self.transmission_reference = transmission_reference

    def set_result_code(self, result_code):
        self.result_code = result_code

    def set_total_records(self, total_records):
        self.total_records = total_records

    # Getters
    def get_version(self):
        return self.version if self.version else ""

    def get_transmission_reference(self):
        return self.transmission_reference if self.transmission_reference else ""

    def get_result_code(self):
        return self.result_code if self.result_code else ""

    def get_total_records(self):
        return self.total_records if self.total_records else ""
    

class PeopleBusinessSearchResult:
    def __init__(self, match_level="", address=None, consumer=None, phone=None):
        self.match_level = match_level
        self.address = address
        self.consumer = consumer
        self.phone = phone

    @classmethod
    def from_dict(cls, data):
        cls.data = data
        address = PeopleBusinessSearchAddress.from_dict(data.get('Address')) if 'Address' in data else None
        consumer = PeopleBusinessSearchConsumer.from_dict(data.get('Consumer')) if 'Consumer' in data else None
        phone = PeopleBusinessSearchPhone.from_dict(data.get('Phone')) if 'Phone' in data else None

        return cls(
            match_level=data.get('matchLevel'),
            address=address,
            consumer=consumer,
            phone=phone
        )
    
    # Setters

    def set_match_level(self, match_level):
        self.match_level = match_level

    # Getters

    def get_match_level(self):
        return self.get_match_level if self.get_match_level else ""
    
class PeopleBusinessSearchAddress:
    def __init__(self, address1="", dependent_locality="", locality="", locality_alternates="",
                 administrative_area="", country_code="", country_name="", thoroughfare="",
                 thoroughfare_pre_direction="", thoroughfare_name="", thoroughfare_trailing_type="",
                 thoroughfare_post_direction="", premises="", premise_type="", premise_number="",
                 sub_premises="", sub_premise_type="", sub_premise_number="", postal_code="",
                 postal_code_secondary="", melissa_address_key="", melissa_address_key_base=""):
        self.address1 = address1
        self.dependent_locality = dependent_locality
        self.locality = locality
        self.locality_alternates = locality_alternates
        self.administrative_area = administrative_area
        self.country_code = country_code
        self.country_name = country_name
        self.thoroughfare = thoroughfare
        self.thoroughfare_pre_direction = thoroughfare_pre_direction
        self.thoroughfare_name = thoroughfare_name
        self.thoroughfare_trailing_type = thoroughfare_trailing_type
        self.thoroughfare_post_direction = thoroughfare_post_direction
        self.premises = premises
        self.premise_type = premise_type
        self.premise_number = premise_number
        self.sub_premises = sub_premises
        self.sub_premise_type = sub_premise_type
        self.sub_premise_number = sub_premise_number
        self.postal_code = postal_code
        self.postal_code_secondary = postal_code_secondary
        self.melissa_address_key = melissa_address_key
        self.melissa_address_key_base = melissa_address_key_base

    @classmethod
    def from_dict(cls, data):
        cls.data = data
        return cls(
            address1=data.get('Address1'),
            dependent_locality=data.get('DependentLocality'),
            locality=data.get('Locality'),
            locality_alternates=data.get('LocalityAlternates'),
            administrative_area=data.get('AdministrativeArea'),
            country_code=data.get('CountryCode'),
            country_name=data.get('CountryName'),
            thoroughfare=data.get('Thoroughfare'),
            thoroughfare_pre_direction=data.get('ThoroughfarePreDirection'),
            thoroughfare_name=data.get('ThoroughfareName'),
            thoroughfare_trailing_type=data.get('ThoroughfareTrailingType'),
            thoroughfare_post_direction=data.get('ThoroughfarePostDirection'),
            premises=data.get('Premises'),
            premise_type=data.get('PremiseType'),
            premise_number=data.get('PremiseNumber'),
            sub_premises=data.get('SubPremises'),
            sub_premise_type=data.get('SubPremiseType'),
            sub_premise_number=data.get('SubPremiseNumber'),
            postal_code=data.get('PostalCode'),
            postal_code_secondary=data.get('PostalCodeSecondary'),
            melissa_address_key=data.get('MelissaAddressKey'),
            melissa_address_key_base=data.get('MelissaAddressKeyBase')
        )
    
    class PeopleBusinessSearchAddress:
        def __init__(self):
            self._address1 = None
            self._dependent_locality = None
            self._locality = None
            self._locality_alternates = None
            self._administrative_area = None
            self._country_code = None
            self._country_name = None
            self._thoroughfare = None
            self._thoroughfare_pre_direction = None
            self._thoroughfare_name = None
            self._thoroughfare_trailing_type = None
            self._thoroughfare_post_direction = None
            self._premises = None
            self._premise_type = None
            self._premise_number = None
            self._sub_premises = None
            self._sub_premise_type = None
            self._sub_premise_number = None
            self._postal_code = None
            self._postal_code_secondary = None
            self._melissa_address_key = None
            self._melissa_address_key_base = None

    # Setters
    def set_address1(self, address1):
        self._address1 = address1

    def set_dependent_locality(self, dependent_locality):
        self._dependent_locality = dependent_locality

    def set_locality(self, locality):
        self._locality = locality

    def set_locality_alternates(self, locality_alternates):
        self._locality_alternates = locality_alternates

    def set_administrative_area(self, administrative_area):
        self._administrative_area = administrative_area

    def set_country_code(self, country_code):
        self._country_code = country_code

    def set_country_name(self, country_name):
        self._country_name = country_name

    def set_thoroughfare(self, thoroughfare):
        self._thoroughfare = thoroughfare

    def set_thoroughfare_pre_direction(self, thoroughfare_pre_direction):
        self._thoroughfare_pre_direction = thoroughfare_pre_direction

    def set_thoroughfare_name(self, thoroughfare_name):
        self._thoroughfare_name = thoroughfare_name

    def set_thoroughfare_trailing_type(self, thoroughfare_trailing_type):
        self._thoroughfare_trailing_type = thoroughfare_trailing_type

    def set_thoroughfare_post_direction(self, thoroughfare_post_direction):
        self._thoroughfare_post_direction = thoroughfare_post_direction

    def set_premises(self, premises):
        self._premises = premises

    def set_premise_type(self, premise_type):
        self._premise_type = premise_type

    def set_premise_number(self, premise_number):
        self._premise_number = premise_number

    def set_sub_premises(self, sub_premises):
        self._sub_premises = sub_premises

    def set_sub_premise_type(self, sub_premise_type):
        self._sub_premise_type = sub_premise_type

    def set_sub_premise_number(self, sub_premise_number):
        self._sub_premise_number = sub_premise_number

    def set_postal_code(self, postal_code):
        self._postal_code = postal_code

    def set_postal_code_secondary(self, postal_code_secondary):
        self._postal_code_secondary = postal_code_secondary

    def set_melissa_address_key(self, melissa_address_key):
        self._melissa_address_key = melissa_address_key

    def set_melissa_address_key_base(self, melissa_address_key_base):
        self._melissa_address_key_base = melissa_address_key_base

    # Getters
    def get_address1(self):
        return self._address1 if self._address1 else ""

    def get_dependent_locality(self):
        return self._dependent_locality if self._dependent_locality else ""

    def get_locality(self):
        return self._locality if self._locality else ""

    def get_locality_alternates(self):
        return self._locality_alternates if self._locality_alternates else ""

    def get_administrative_area(self):
        return self._administrative_area if self._administrative_area else ""

    def get_country_code(self):
        return self._country_code if self._country_code else ""

    def get_country_name(self):
        return self._country_name if self._country_name else ""

    def get_thoroughfare(self):
        return self._thoroughfare if self._thoroughfare else ""

    def get_thoroughfare_pre_direction(self):
        return self._thoroughfare_pre_direction if self._thoroughfare_pre_direction else ""

    def get_thoroughfare_name(self):
        return self._thoroughfare_name if self._thoroughfare_name else ""

    def get_thoroughfare_trailing_type(self):
        return self._thoroughfare_trailing_type if self._thoroughfare_trailing_type else ""

    def get_thoroughfare_post_direction(self):
        return self._thoroughfare_post_direction if self._thoroughfare_post_direction else ""

    def get_premises(self):
        return self._premises if self._premises else ""

    def get_premise_type(self):
        return self._premise_type if self._premise_type else ""

    def get_premise_number(self):
        return self._premise_number if self._premise_number else ""

    def get_sub_premises(self):
        return self._sub_premises if self._sub_premises else ""

    def get_sub_premise_type(self):
        return self._sub_premise_type if self._sub_premise_type else ""

    def get_sub_premise_number(self):
        return self._sub_premise_number if self._sub_premise_number else ""

    def get_postal_code(self):
        return self._postal_code if self._postal_code else ""

    def get_postal_code_secondary(self):
        return self._postal_code_secondary if self._postal_code_secondary else ""

    def get_melissa_address_key(self):
        return self._melissa_address_key if self._melissa_address_key else ""

    def get_melissa_address_key_base(self):
        return self._melissa_address_key_base if self._melissa_address_key_base else ""
    
class PeopleBusinessSearchConsumer:
    def __init__(self, full_name="", first_name="", middle_name="", last_name="", suffix="", other_full_names="", melissa_identity_key=""):
        self.full_name = full_name
        self.first_name = first_name
        self.middle_name = middle_name
        self.last_name = last_name
        self.suffix = suffix
        self.other_full_names = other_full_names
        self.melissa_identity_key = melissa_identity_key

    @classmethod
    def from_dict(cls, data):
        cls.data = data
        return cls(
            full_name=data.get("FullName", ""),
            first_name=data.get("FirstName", ""),
            middle_name=data.get("MiddleName", ""),
            last_name=data.get("LastName", ""),
            suffix=data.get("Suffix", ""),
            other_full_names=data.get("OtherFullNames", ""),
            melissa_identity_key=data.get("MelissaIdentityKey", "")
        )
    

    # Setters
    
    def set_full_name(self, full_name):
        self.full_name = full_name

    def set_first_name(self, first_name):
        self.first_name = first_name

    def set_middle_name(self, middle_name):
        self.middle_name = middle_name

    def set_last_name(self, last_name):
        self.last_name = last_name

    def set_suffix(self, suffix):
        self.suffix = suffix

    def set_other_full_names(self, other_full_names):
        self.other_full_names = other_full_names

    def set_melissa_identity_key(self, melissa_identity_key):
        self.melissa_identity_key = melissa_identity_key

    # Getters

    def get_full_name(self):
        return self.full_name or ""

    def get_first_name(self):
        return self.first_name or ""

    def get_middle_name(self):
        return self.middle_name or ""

    def get_last_name(self):
        return self.last_name or ""

    def get_suffix(self):
        return self.suffix or ""

    def get_other_full_names(self):
        return self.other_full_names or ""

    def get_melissa_identity_key(self):
        return self.melissa_identity_key or ""

class PeopleBusinessSearchPhone:
    def __init__(self, phone="", other_phones=""):
        self.phone = phone
        self.other_phones = other_phones

    @classmethod
    def from_dict(cls, data):
        cls.data = data
        return cls(
            phone=data.get("Phone", ""),
            other_phones=data.get("OtherPhones", "")
        )
    
    
    # Setter methods
    
    def set_phone(self, phone):
        self.phone = phone

    def set_other_phones(self, other_phones):
        self.other_phones = other_phones

    # Getter methods
    
    def get_phone(self):
        return self.phone or ""

    def get_other_phones(self):
        return self.other_phones or ""


    


import json
import os
from .CloudApiBase import CloudApiBase
from ..PostReqestBase import SSNNameMatchPostRequest


class SSNNameMatch(CloudApiBase):

    def __init__(self, license=None):
        super().__init__("https://namessn.melissadata.net", "/v4/web/SSN/doLookup")
        self.license = license or os.environ.get("MD_LICENSE")
        self.parameters["format"] = "json"
        self.records = []
        self.post_body = None

        self.parameter_mappings = {
            "ssn": "ssn",
            "first_name": "first",
            "last_name": "last",
            "full_name": "full",
            "transmission_reference": "t"
        }

        # Initialize private backing fields
        for attr in self.parameter_mappings.keys():
            setattr(self, f"_{attr}", None)

    def _set_parameter(self, attr_name, value):
        """
        Helper method to set a private attribute and update the parameters dictionary.
        """
        setattr(self, f"_{attr_name}", value)
        self.parameters[self.parameter_mappings[attr_name]] = value

    # Properties
    @property
    def ssn(self):
        return self._ssn

    @ssn.setter
    def ssn(self, value):
        self._set_parameter("ssn", value)

    @property
    def first_name(self):
        return self._first_name

    @first_name.setter
    def first_name(self, value):
        self._set_parameter("first_name", value)

    @property
    def last_name(self):
        return self._last_name

    @last_name.setter
    def last_name(self, value):
        self._set_parameter("last_name", value)

    @property
    def full_name(self):
        return self._full_name

    @full_name.setter
    def full_name(self, value):
        self._set_parameter("full_name", value)

    @property
    def transmission_reference(self):
        return self._transmission_reference

    @transmission_reference.setter
    def transmission_reference(self, value):
        self._set_parameter("transmission_reference", value)

    # Setters
    def set_ssn(self, ssn):
        self._set_parameter("ssn", ssn)

    def set_first_name(self, first_name):
        self._set_parameter("first_name", first_name)

    def set_last_name(self, last_name):
        self._set_parameter("last_name", last_name)

    def set_full_name(self, full_name):
        self._set_parameter("full_name", full_name)

    def set_transmission_reference(self, transmission_reference):
        self._set_parameter("transmission_reference", transmission_reference)

    def set_post_body(self, post_body):
        self.post_body = post_body

    def set_value(self, parameter, value):
        parameter = parameter.strip() if parameter else None
        
        if parameter in self.parameter_mappings:
            parameter_key = self.parameter_mappings[parameter]
            
            property_name = next((key for key, val in self.parameter_mappings.items() 
                                  if val.lower() == parameter_key.lower()), None)
            
            if property_name:
                if hasattr(self, property_name):
                    setattr(self, property_name, value)
        else:
            # If not in derived class, go to base class implementation
            super().set_value(parameter, value)

    # Getters
    def get_ssn(self):
        return self.parameters.get("ssn", "")

    def get_first_name(self):
        return self.parameters.get("first", "")

    def get_last_name(self):
        return self.parameters.get("last", "")

    def get_full_name(self):
        return self.parameters.get("full", "")

    def get_transmission_reference(self):
        return self.parameters.get("t", "")

    def get_post_body(self):
        return self.post_body
    
    # Class Methods
    def add_record(self, request):
        self.records.append(request)
    
    def clear_records(self):
        self.records.clear()

    def post(self, response_type):
        """
        This synchronous function makes the batch request using the post body.

        Returns:
            A string or deserialized json response object.
        """

        if self.records and len(self.records) > 0:
            post_request = SSNNameMatchPostRequest(
                transmission_reference=self.get_transmission_reference(),
                records=self.records,
                customer_id=self.get_license()
            )
            response = self.send_post_request(post_request)
        else:
            response = self.send_post_request(self.post_body)
        return self.handle_response(response, response_type)


    def get_api_version(self):
        """
        Makes a synchronous getversion request and parses the response for the API version.

        Returns:
            The Cloud API version.
        """
        json_response = self.send_version_request()
        response_objectect = json.loads(json_response)
        version = response_objectect["Version"]
        return version



from typing import List
from .ResponseBase import ResponseBase

class BusinessCoderResponse(ResponseBase):
    def __init__(self, 
                 version = "", 
                 transmission_reference = "", 
                 transmission_results = "", 
                 total_records = "", 
                 records: list = None):
        self.version = version
        self.transmission_reference = transmission_reference
        self.transmission_results = transmission_results
        self.total_records = total_records
        self.records = records if records is not None else []

    @classmethod
    def populate_from_dict(cls, data: dict):
        records = [BusinessCoderRecord.from_dict(record) for record in data.get("Records", [])]
        cls.data = data
        return cls(
            version=data.get("Version", ""),
            transmission_reference=data.get("TransmissionReference", ""),
            transmission_results=data.get("TransmissionResults", ""),
            total_records=data.get("TotalRecords", ""),
            records=records
        )

    # Setters
    def set_version(self, version):
        self.version = version

    def set_transmission_reference(self, transmission_reference):
        self.transmission_reference = transmission_reference

    def set_transmission_results(self, transmission_results):
        self.transmission_results = transmission_results

    def set_total_records(self, total_records):
        self.total_records = total_records

    # Getters
    def get_version(self):
        return self.version or ""

    def get_transmission_reference(self):
        return self.transmission_reference or ""

    def get_transmission_results(self):
        return self.transmission_results or ""

    def get_total_records(self):
        return self.total_records or ""
    

class BusinessCoderContacts(ResponseBase):
    def __init__(self, 
                 name_first = "", 
                 name_last = "", 
                 gender = "", 
                 title = "", 
                 contact_phone = "", 
                 email = ""):
        self.name_first = name_first
        self.name_last = name_last
        self.gender = gender
        self.title = title
        self.contact_phone = contact_phone
        self.email = email

    @classmethod
    def from_dict(cls, data: dict):
        cls.data = data
        return cls(
            name_first=data.get('NameFirst', ""),
            name_last=data.get('NameLast', ""),
            gender=data.get('Gender', ""),
            title=data.get('Title', ""),
            contact_phone=data.get('ContactPhone', ""),
            email=data.get('Email', "")
        )

    # Setters
    def set_name_first(self, name_first):
        self.name_first = name_first

    def set_name_last(self, name_last):
        self.name_last = name_last

    def set_gender(self, gender):
        self.gender = gender

    def set_title(self, title):
        self.title = title

    def set_contact_phone(self, contact_phone):
        self.contact_phone = contact_phone

    def set_email(self, email):
        self.email = email

    # Getters
    def get_name_first(self):
        return self.name_first or ""

    def get_name_last(self):
        return self.name_last or ""

    def get_gender(self):
        return self.gender or ""

    def get_title(self):
        return self.title or ""

    def get_contact_phone(self):
        return self.contact_phone or ""

    def get_email(self):
        return self.email or ""

    


class BusinessCoderRecord(ResponseBase):
    def __init__(self, 
             record_id="", 
             company_name="", 
             current_company_name="", 
             address_line_1="", 
             suite="", 
             city="", 
             state="", 
             postal_code="", 
             total_contacts="", 
             melissa_enterprise_key="", 
             location_type="", 
             phone="", 
             employees_estimate="", 
             sales_estimate="", 
             stock_ticker="", 
             web_address="", 
             contacts: List[BusinessCoderContacts] = None, 
             country_code="", 
             country_name="", 
             delivery_indicator="", 
             melissa_address_key="", 
             melissa_address_key_base="", 
             plus4="", 
             ein="", 
             sic_code1="", 
             sic_code2="", 
             sic_code3="", 
             naics_code1="", 
             naics_code2="", 
             naics_code3="", 
             sic_description1="", 
             sic_description2="", 
             sic_description3="", 
             naics_description1="", 
             naics_description2="", 
             naics_description3="", 
             latitude="", 
             longitude="", 
             county_name="", 
             county_fips="", 
             census_tract="", 
             census_block="", 
             place_code="", 
             place_name="", 
             results=""):
        self.record_id = record_id
        self.company_name = company_name
        self.current_company_name = current_company_name
        self.address_line_1 = address_line_1
        self.suite = suite
        self.city = city
        self.state = state
        self.postal_code = postal_code
        self.total_contacts = total_contacts
        self.melissa_enterprise_key = melissa_enterprise_key
        self.location_type = location_type
        self.phone = phone
        self.employees_estimate = employees_estimate
        self.sales_estimate = sales_estimate
        self.stock_ticker = stock_ticker
        self.web_address = web_address
        self.contacts = contacts if contacts is not None else []
        self.country_code = country_code
        self.country_name = country_name
        self.delivery_indicator = delivery_indicator
        self.melissa_address_key = melissa_address_key
        self.melissa_address_key_base = melissa_address_key_base
        self.plus4 = plus4
        self.ein = ein
        self.sic_code1 = sic_code1
        self.sic_code2 = sic_code2
        self.sic_code3 = sic_code3
        self.naics_code1 = naics_code1
        self.naics_code2 = naics_code2
        self.naics_code3 = naics_code3
        self.sic_description1 = sic_description1
        self.sic_description2 = sic_description2
        self.sic_description3 = sic_description3
        self.naics_description1 = naics_description1
        self.naics_description2 = naics_description2
        self.naics_description3 = naics_description3
        self.latitude = latitude
        self.longitude = longitude
        self.county_name = county_name
        self.county_fips = county_fips
        self.census_tract = census_tract
        self.census_block = census_block
        self.place_code = place_code
        self.place_name = place_name
        self.results = results
    
    @classmethod
    def from_dict(cls, data: dict):
        cls.data = data
        return cls(
            record_id=data.get('RecordID', ""),
            company_name=data.get('CompanyName', ""),
            current_company_name=data.get('CurrentCompanyName', ""),
            address_line_1=data.get('AddressLine1', ""),
            suite=data.get('Suite', ""),
            city=data.get('City', ""),
            state=data.get('State', ""),
            postal_code=data.get('PostalCode', ""),
            total_contacts=data.get('TotalContacts', ""),
            melissa_enterprise_key=data.get('MelissaEnterpriseKey', ""),
            location_type=data.get('LocationType', ""),
            phone=data.get('Phone', ""),
            employees_estimate=data.get('EmployeesEstimate', ""),
            sales_estimate=data.get('SalesEstimate', ""),
            stock_ticker=data.get('StockTicker', ""),
            web_address=data.get('WebAddress', ""),
            contacts=[BusinessCoderContacts.from_dict(data) for data in data.get('Contacts', [])],
            country_code=data.get('CountryCode', ""),
            country_name=data.get('CountryName', ""),
            delivery_indicator=data.get('DeliveryIndicator', ""),
            melissa_address_key=data.get('MelissaAddressKey', ""),
            melissa_address_key_base=data.get('MelissaAddressKeyBase', ""),
            plus4=data.get('Plus4', ""),
            ein=data.get('EIN', ""),
            sic_code1=data.get('SICCode1', ""),
            sic_code2=data.get('SICCode2', ""),
            sic_code3=data.get('SICCode3', ""),
            naics_code1=data.get('NAICSCode1', ""),
            naics_code2=data.get('NAICSCode2', ""),
            naics_code3=data.get('NAICSCode3', ""),
            sic_description1=data.get('SICDescription1', ""),
            sic_description2=data.get('SICDescription2', ""),
            sic_description3=data.get('SICDescription3', ""),
            naics_description1=data.get('NAICSDescription1', ""),
            naics_description2=data.get('NAICSDescription2', ""),
            naics_description3=data.get('NAICSDescription3', ""),
            latitude=data.get('Latitude', ""),
            longitude=data.get('Longitude', ""),
            county_name=data.get('CountyName', ""),
            county_fips=data.get('CountyFIPS', ""),
            census_tract=data.get('CensusTract', ""),
            census_block=data.get('CensusBlock', ""),
            place_code=data.get('PlaceCode', ""),
            place_name=data.get('PlaceName', ""),
            results=data.get('Results', "")
        )

    # Setters
    def set_record_id(self, record_id):
       self.record_id = record_id

    def set_company_name(self, company_name):
        self.company_name = company_name

    def set_current_company_name(self, current_company_name):
        self.current_company_name = current_company_name

    def set_address_line_1(self, address_line_1):
        self.address_line_1 = address_line_1

    def set_suite(self, suite):
        self.suite = suite

    def set_city(self, city):
        self.city = city

    def set_state(self, state):
        self.state = state

    def set_postal_code(self, postal_code):
        self.postal_code = postal_code

    def set_total_contacts(self, total_contacts):
        self.total_contacts = total_contacts

    def set_melissa_enterprise_key(self, melissa_enterprise_key):
        self.melissa_enterprise_key = melissa_enterprise_key

    def set_location_type(self, location_type):
        self.location_type = location_type

    def set_phone(self, phone):
        self.phone = phone

    def set_employees_estimate(self, employees_estimate):
        self.employees_estimate = employees_estimate

    def set_sales_estimate(self, sales_estimate):
        self.sales_estimate = sales_estimate

    def set_stock_ticker(self, stock_ticker):
        self.stock_ticker = stock_ticker

    def set_web_address(self, web_address):
        self.web_address = web_address

    def set_country_code(self, country_code):
        self.country_code = country_code

    def set_country_name(self, country_name):
        self.country_name = country_name

    def set_delivery_indicator(self, delivery_indicator):
        self.delivery_indicator = delivery_indicator

    def set_melissa_address_key(self, melissa_address_key):
        self.melissa_address_key = melissa_address_key

    def set_melissa_address_key_base(self, melissa_address_key_base):
        self.melissa_address_key_base = melissa_address_key_base

    def set_plus4(self, plus4):
        self.plus4 = plus4

    def set_ein(self, ein):
        self.ein = ein

    def set_sic_code1(self, sic_code1):
        self.sic_code1 = sic_code1

    def set_sic_code2(self, sic_code2):
        self.sic_code2 = sic_code2

    def set_sic_code3(self, sic_code3):
        self.sic_code3 = sic_code3

    def set_naics_code1(self, naics_code1):
        self.naics_code1 = naics_code1

    def set_naics_code2(self, naics_code2):
        self.naics_code2 = naics_code2

    def set_naics_code3(self, naics_code3):
        self.naics_code3 = naics_code3

    def set_sic_description1(self, sic_description1):
        self.sic_description1 = sic_description1

    def set_sic_description2(self, sic_description2):
        self.sic_description2 = sic_description2

    def set_sic_description3(self, sic_description3):
        self.sic_description3 = sic_description3

    def set_naics_description1(self, naics_description1):
        self.naics_description1 = naics_description1

    def set_naics_description2(self, naics_description2):
        self.naics_description2 = naics_description2

    def set_naics_description3(self, naics_description3):
        self.naics_description3 = naics_description3

    def set_latitude(self, latitude):
        self.latitude = latitude

    def set_longitude(self, longitude):
        self.longitude = longitude

    def set_county_name(self, county_name):
        self.county_name = county_name

    def set_county_fips(self, county_fips):
        self.county_fips = county_fips

    def set_census_tract(self, census_tract):
        self.census_tract = census_tract

    def set_census_block(self, census_block):
        self.census_block = census_block

    def set_place_code(self, place_code):
        self.place_code = place_code

    def set_place_name(self, place_name):
        self.place_name = place_name

    def set_results(self, results):
        self.results = results

    # Getters
    def get_record_id(self):
        return self.record_id if self.record_id else ""

    def get_company_name(self):
        return self.company_name if self.company_name else ""

    def get_current_company_name(self):
        return self.current_company_name if self.current_company_name else ""

    def get_address_line_1(self):
        return self.address_line_1 if self.address_line_1 else ""

    def get_suite(self):
        return self.suite if self.suite else ""

    def get_city(self):
        return self.city if self.city else ""

    def get_state(self):
        return self.state if self.state else ""

    def get_postal_code(self):
        return self.postal_code if self.postal_code else ""

    def get_total_contacts(self):
        return self.total_contacts if self.total_contacts else ""

    def get_melissa_enterprise_key(self):
        return self.melissa_enterprise_key if self.melissa_enterprise_key else ""

    def get_location_type(self):
        return self.location_type if self.location_type else ""

    def get_phone(self):
        return self.phone if self.phone else ""

    def get_employees_estimate(self):
        return self.employees_estimate if self.employees_estimate else ""

    def get_sales_estimate(self):
        return self.sales_estimate if self.sales_estimate else ""

    def get_stock_ticker(self):
        return self.stock_ticker if self.stock_ticker else ""

    def get_web_address(self):
        return self.web_address if self.web_address else ""

    def get_country_code(self):
        return self.country_code if self.country_code else ""

    def get_country_name(self):
        return self.country_name if self.country_name else ""

    def get_delivery_indicator(self):
        return self.delivery_indicator if self.delivery_indicator else ""

    def get_melissa_address_key(self):
        return self.melissa_address_key if self.melissa_address_key else ""

    def get_melissa_address_key_base(self):
        return self.melissa_address_key_base if self.melissa_address_key_base else ""

    def get_plus4(self):
        return self.plus4 if self.plus4 else ""

    def get_ein(self):
        return self.ein if self.ein else ""

    def get_sic_code1(self):
        return self.sic_code1 if self.sic_code1 else ""

    def get_sic_code2(self):
        return self.sic_code2 if self.sic_code2 else ""

    def get_sic_code3(self):
        return self.sic_code3 if self.sic_code3 else ""

    def get_naics_code1(self):
        return self.naics_code1 if self.naics_code1 else ""

    def get_naics_code2(self):
        return self.naics_code2 if self.naics_code2 else ""

    def get_naics_code3(self):
        return self.naics_code3 if self.naics_code3 else ""

    def get_sic_description1(self):
        return self.sic_description1 if self.sic_description1 else ""

    def get_sic_description2(self):
        return self.sic_description2 if self.sic_description2 else ""

    def get_sic_description3(self):
        return self.sic_description3 if self.sic_description3 else ""

    def get_naics_description1(self):
        return self.naics_description1 if self.naics_description1 else ""

    def get_naics_description2(self):
        return self.naics_description2 if self.naics_description2 else ""

    def get_naics_description3(self):
        return self.naics_description3 if self.naics_description3 else ""

    def get_latitude(self):
        return self.latitude if self.latitude else ""

    def get_longitude(self):
        return self.longitude if self.longitude else ""

    def get_county_name(self):
        return self.county_name if self.county_name else ""

    def get_county_fips(self):
        return self.county_fips if self.county_fips else ""

    def get_census_tract(self):
        return self.census_tract if self.census_tract else ""

    def get_census_block(self):
        return self.census_block if self.census_block else ""

    def get_place_code(self):
        return self.place_code if self.place_code else ""

    def get_place_name(self):
        return self.place_name if self.place_name else ""

    def get_results(self):
        return self.results if self.results else ""
    




import json
import os
from .CloudApiBase import CloudApiBase
from ..PostReqestBase import PropertyPostRequest


class Property(CloudApiBase):

    def __init__(self, license=None):
        super().__init__("https://property.melissadata.net", "/v4/WEB/LookupProperty")
        self.license = license or os.environ.get("MD_LICENSE")
        self.parameters["format"] = "json"
        self.records = []
        self.post_body = None

        self.parameter_mappings = {
            "address_key": "addresskey",
            "address_line_1": "a1",
            "address_line_2": "a2",
            "apn": "apn",
            "city": "city",
            "country": "ctry",
            "fips": "fips",
            "free_form": "ff",
            "mak": "mak",
            "state": "state",
            "postal": "postal",
            "transmission_reference": "t",
            "cols": "cols",
            "opt": "opt",
            "owner_mak": "mak",
            "total_records": "totalRecords",
        }

        # Initialize private backing fields
        for attr in self.parameter_mappings.keys():
            setattr(self, f"_{attr}", None)

    def _set_parameter(self, attr_name, value):
        """
        Helper method to set a private attribute and update the parameters dictionary.
        """
        setattr(self, f"_{attr_name}", value)
        self.parameters[self.parameter_mappings[attr_name]] = value

    # Properties
    @property
    def address_key(self):
        return self._address_key

    @address_key.setter
    def address_key(self, value):
        self._set_parameter("address_key", value)

    @property
    def address_line_1(self):
        return self._address_line_1

    @address_line_1.setter
    def address_line_1(self, value):
        self._set_parameter("address_line_1", value)

    @property
    def address_line_2(self):
        return self._address_line_2

    @address_line_2.setter
    def address_line_2(self, value):
        self._set_parameter("address_line_2", value)

    @property
    def apn(self):
        return self._apn

    @apn.setter
    def apn(self, value):
        self._set_parameter("apn", value)

    @property
    def city(self):
        return self._city

    @city.setter
    def city(self, value):
        self._set_parameter("city", value)

    @property
    def country(self):
        return self._country

    @country.setter
    def country(self, value):
        self._set_parameter("country", value)

    @property
    def fips(self):
        return self._fips

    @fips.setter
    def fips(self, value):
        self._set_parameter("fips", value)

    @property
    def free_form(self):
        return self._free_form

    @free_form.setter
    def free_form(self, value):
        self._set_parameter("free_form", value)

    @property
    def mak(self):
        return self._mak

    @mak.setter
    def mak(self, value):
        self._set_parameter("mak", value)

    @property
    def state(self):
        return self._state

    @state.setter
    def state(self, value):
        self._set_parameter("state", value)

    @property
    def postal(self):
        return self._postal

    @postal.setter
    def postal(self, value):
        self._set_parameter("postal", value)

    @property
    def transmission_reference(self):
        return self._transmission_reference

    @transmission_reference.setter
    def transmission_reference(self, value):
        self._set_parameter("transmission_reference", value)

    @property
    def cols(self):
        return self._cols

    @cols.setter
    def cols(self, value):
        self._set_parameter("cols", value)

    @property
    def opt(self):
        return self._opt

    @opt.setter
    def opt(self, value):
        self._set_parameter("opt", value)

    @property
    def owner_mak(self):
        return self._owner_mak

    @owner_mak.setter
    def owner_mak(self, value):
        self._set_parameter("owner_mak", value)

    @property
    def total_records(self):
        return self._total_records

    @total_records.setter
    def total_records(self, value):
        self._set_parameter("total_records", value)


    # Setters
    def set_address_key(self, address_key):
        self._set_parameter("address_key", address_key)

    def set_address_line_1(self, address_line_1):
        self._set_parameter("address_line_1", address_line_1)

    def set_address_line_2(self, address_line_2):
        self._set_parameter("address_line_2", address_line_2)

    def set_apn(self, apn):
        self._set_parameter("apn", apn)

    def set_city(self, city):
        self._set_parameter("city", city)

    def set_country(self, country):
        self._set_parameter("country", country)

    def set_fips(self, fips):
        self._set_parameter("fips", fips)

    def set_free_form(self, free_form):
        self._set_parameter("free_form", free_form)

    def set_mak(self, mak):
        self._set_parameter("mak", mak)

    def set_state(self, state):
        self._set_parameter("state", state)

    def set_postal(self, postal):
        self._set_parameter("postal", postal)

    def set_transmission_reference(self, transmission_reference):
        self._set_parameter("transmission_reference", transmission_reference)

    def set_cols(self, cols):
        self._set_parameter("cols", cols)

    def set_opt(self, opt):
        self._set_parameter("opt", opt)

    def set_owner_mak(self, owner_mak):
        self._set_parameter("owner_mak", owner_mak)

    def set_total_records(self, total_records):
        self._set_parameter("total_records", total_records)

    def set_post_body(self, post_body):
        self._set_parameter("post_body", post_body)

    def set_post_body(self, post_body):
        self.post_body = post_body
        self.post_body = post_body

    def set_value(self, parameter, value):
        parameter = parameter.strip() if parameter else None
        
        if parameter in self.parameter_mappings:
            parameter_key = self.parameter_mappings[parameter]
            
            property_name = next((key for key, val in self.parameter_mappings.items() 
                                  if val.lower() == parameter_key.lower()), None)
            
            if property_name:
                if hasattr(self, property_name):
                    setattr(self, property_name, value)
        else:
            # If not in derived class, go to base class implementation
            super().set_value(parameter, value)

    # Getters
    def get_address_key(self):
        return self.parameters.get("addresskey", "")

    def get_address_line_1(self):
        return self.parameters.get("a1", "")

    def get_address_line_2(self):
        return self.parameters.get("a2", "")

    def get_apn(self):
        return self.parameters.get("apn", "")

    def get_city(self):
        return self.parameters.get("city", "")

    def get_country(self):
        return self.parameters.get("ctry", "")

    def get_fips(self):
        return self.parameters.get("fips", "")

    def get_free_form(self):
        return self.parameters.get("ff", "")

    def get_mak(self):
        return self.parameters.get("mak", "")

    def get_state(self):
        return self.parameters.get("state", "")

    def get_postal(self):
        return self.parameters.get("postal", "")

    def get_transmission_reference(self):
        return self.parameters.get("t", "")

    def get_cols(self):
        return self.parameters.get("cols", "")

    def get_opt(self):
        return self.parameters.get("opt", "")

    def get_owner_mak(self):
        return self.parameters.get("mak", "")

    def get_total_records(self):
        return self.parameters.get("totalRecords", "")

    def get_post_batch_body(self):
        return self.post_body


# Class Methods
    def add_record(self, request):
        self.records.append(request)

    def clear_records(self):
        self.records.clear()

    def get_lookup_property(self, response_type):
        """
        This synchronous function sends the CloudAPI request using the /v4/WEB/LookupProperty endpoint
        """
        self.set_endpoint("/v4/WEB/LookupProperty")
        return self.get(response_type)


    def post_lookup_property(self, response_type):
        """
        This synchronous function sends a POST request to the Cloud API using the /v4/WEB/LookupProperty endpoint
        """
        self.set_endpoint("/v4/WEB/LookupProperty")

        if self.records:
            post_request = PropertyPostRequest(
                columns=self.get_cols(),
                customer_id=self.get_license(),
                transmission_reference=self.get_transmission_reference(),
                options=self.get_opt(),
                total_records=self.get_total_records(),
                records=self.records,
            )
            response = self.send_post_request(post_request)
        else:
            response = self.send_post_request(self.post_body)

        return self.handle_response(response, response_type)

    
    def get_lookup_deeds(self, response_type):
        """
        This synchronous function sends the CloudAPI request using the /v4/WEB/LookupDeeds endpoint
        """
        self.set_endpoint("/v4/WEB/LookupDeeds")
        return self.get(response_type)

    def post_lookup_deeds(self, response_type):
        """
        This synchronous function sends a POST request to the Cloud API using the /v4/WEB/LookupDeeds endpoint
        """
        self.set_endpoint("/v4/WEB/LookupDeeds")
        response = self.send_post_request(self.post_body)
        return self.handle_response(response, response_type)
    
    def get_lookup_homes_by_owner(self, response_type):
        """
        This synchronous function sends the CloudAPI request using the /v4/WEB/LookupHomesByOwner endpoint
        """
        self.set_endpoint("/v4/WEB/LookupHomesByOwner")
        return self.get(response_type)

    def post_lookup_homes_by_owner(self, response_type):
        """
        This synchronous function sends a POST request to the Cloud API using the /v4/WEB/LookupHomesByOwner endpoint
        """
        self.set_endpoint("/v4/WEB/LookupHomesByOwner")
        response = self.send_post_request(self.post_body)
        return self.handle_response(response, response_type)
    
    def get_api_version(self):
        """
        Makes a synchronous getversion request and parses the response for the API version.

        Returns:
            The Cloud API version.
        """
        json_response = self.send_version_request()
        response_objectect = json.loads(json_response)
        version = response_objectect["Version"]
        return version


from .ResponseBase import ResponseBase

class GlobalPhoneResponse(ResponseBase):
    def __init__(self, version="", transmission_reference="", transmission_results="", total_records="", records=None):
        self.version = version
        self.transmission_reference = transmission_reference
        self.transmission_results = transmission_results
        self.total_records = total_records
        self.records = records if records is not None else []

    @classmethod
    def populate_from_dict(cls, data: dict):
        records = [GlobalPhoneRecord.from_dict(record) for record in data.get("Records", [])]
        return cls(
            version=data.get("Version", ""),
            transmission_reference=data.get("TransmissionReference", ""),
            transmission_results=data.get("TransmissionResults", ""),
            total_records = data.get("TotalRecords", ""),
            records=records
        )
    
    # Setters
    def set_version(self, version):
        self.version = version

    def set_transmission_reference(self, transmission_reference):
        self.transmission_reference = transmission_reference

    def set_transmission_results(self, transmission_results):
        self.transmission_results = transmission_results

    def set_total_records(self, total_records):
        self.total_records = total_records

    # Getters
    def get_version(self):
        return self.version or ""

    def get_transmission_reference(self):
        return self.transmission_reference or ""

    def get_transmission_results(self):
        return self.transmission_results or ""

    def get_total_records(self):
        return self.total_records or ""
    
class GlobalPhoneRecord:
    def __init__(
        self, record_id="", results="", phone_number="", administrative_area="", country_abbreviation="", country_name="", carrier="",
        caller_id="", dst="", international_phone_number="", language="", latitude="", locality="", longitude="", phone_international_prefix="", phone_country_dialing_code="",
        phone_nation_prefix="", phone_national_destination_code="", phone_subscriber_number="", utc="", postal_code="", suggestions="", time_zone_code="", time_zone_name=""
    ):
        self.record_id = record_id
        self.results = results
        self.phone_number = phone_number
        self.administrative_area = administrative_area
        self.country_abbreviation = country_abbreviation
        self.country_name = country_name
        self.carrier = carrier
        self.caller_id = caller_id
        self.dst = dst
        self.international_phone_number = international_phone_number
        self.language = language
        self.latitude = latitude
        self.locality = locality
        self.longitude = longitude
        self.phone_international_prefix = phone_international_prefix
        self.phone_country_dialing_code = phone_country_dialing_code
        self.phone_nation_prefix = phone_nation_prefix
        self.phone_national_destination_code = phone_national_destination_code
        self.phone_subscriber_number = phone_subscriber_number
        self.utc = utc
        self.postal_code = postal_code
        self.suggestions = suggestions
        self.time_zone_code = time_zone_code
        self.time_zone_name = time_zone_name

    @classmethod
    def from_dict(cls, data):
        cls.data = data
        return cls(
            record_id=data.get("RecordID", ""),
            results=data.get("Results", ""),
            phone_number=data.get("PhoneNumber", ""),
            administrative_area=data.get("AdministrativeArea", ""),
            country_abbreviation=data.get("CountryAbbreviation", ""),
            country_name=data.get("CountryName", ""),
            carrier=data.get("Carrier", ""),
            caller_id=data.get("CallerID", ""),
            dst=data.get("DST", ""),
            international_phone_number=data.get("InternationalPhoneNumber", ""),
            language=data.get("Language", ""),
            latitude=data.get("Latitude", ""),
            locality=data.get("Locality", ""),
            longitude=data.get("Longitude", ""),
            phone_international_prefix=data.get("PhoneInternationalPrefix", ""),
            phone_country_dialing_code=data.get("PhoneCountryDialingCode", ""),
            phone_nation_prefix=data.get("PhoneNationPrefix", ""),
            phone_national_destination_code=data.get("PhoneNationalDestinationCode", ""),
            phone_subscriber_number=data.get("PhoneSubscriberNumber", ""),
            utc=data.get("UTC", ""),
            postal_code=data.get("PostalCode", ""),
            suggestions=data.get("Suggestions", ""),
            time_zone_code=data.get("TimeZoneCode", ""),
            time_zone_name=data.get("TimeZoneName", "")
        )
    

    # Setters
    def set_record_id(self, record_id):
        self.record_id = record_id

    def set_results(self, results):
        self.results = results

    def set_phone_number(self, phone_number):
        self.phone_number = phone_number

    def set_administrative_area(self, administrative_area):
        self.administrative_area = administrative_area

    def set_country_abbreviation(self, country_abbreviation):
        self.country_abbreviation = country_abbreviation

    def set_country_name(self, country_name):
        self.country_name = country_name

    def set_carrier(self, carrier):
        self.carrier = carrier

    def set_caller_id(self, caller_id):
        self.caller_id = caller_id

    def set_dst(self, dst):
        self.dst = dst

    def set_international_phone_number(self, international_phone_number):
        self.international_phone_number = international_phone_number

    def set_language(self, language):
        self.language = language

    def set_latitude(self, latitude):
        self.latitude = latitude

    def set_locality(self, locality):
        self.locality = locality

    def set_longitude(self, longitude):
        self.longitude = longitude

    def set_phone_international_prefix(self, phone_international_prefix):
        self.phone_international_prefix = phone_international_prefix

    def set_phone_country_dialing_code(self, phone_country_dialing_code):
        self.phone_country_dialing_code = phone_country_dialing_code

    def set_phone_nation_prefix(self, phone_nation_prefix):
        self.phone_nation_prefix = phone_nation_prefix

    def set_phone_national_destination_code(self, phone_national_destination_code):
        self.phone_national_destination_code = phone_national_destination_code

    def set_phone_subscriber_number(self, phone_subscriber_number):
        self.phone_subscriber_number = phone_subscriber_number

    def set_utc(self, utc):
        self.utc = utc

    def set_postal_code(self, postal_code):
        self.postal_code = postal_code

    def set_suggestions(self, suggestions):
        self.suggestions = suggestions

    def set_time_zone_code(self, time_zone_code):
        self.time_zone_code = time_zone_code

    def set_time_zone_name(self, time_zone_name):
        self.time_zone_name = time_zone_name

    # Getters
    def get_record_id(self):
        return self.record_id or ""

    def get_results(self):
        return self.results or ""

    def get_phone_number(self):
        return self.phone_number or ""

    def get_administrative_area(self):
        return self.administrative_area or ""

    def get_country_abbreviation(self):
        return self.country_abbreviation or ""

    def get_country_name(self):
        return self.country_name or ""

    def get_carrier(self):
        return self.carrier or ""

    def get_caller_id(self):
        return self.caller_id or ""

    def get_dst(self):
        return self.dst or ""

    def get_international_phone_number(self):
        return self.international_phone_number or ""

    def get_language(self):
        return self.language or ""

    def get_latitude(self):
        return self.latitude or ""

    def get_locality(self):
        return self.locality or ""

    def get_longitude(self):
        return self.longitude or ""

    def get_phone_international_prefix(self):
        return self.phone_international_prefix or ""

    def get_phone_country_dialing_code(self):
        return self.phone_country_dialing_code or ""

    def get_phone_nation_prefix(self):
        return self.phone_nation_prefix or ""

    def get_phone_national_destination_code(self):
        return self.phone_national_destination_code or ""

    def get_phone_subscriber_number(self):
        return self.phone_subscriber_number or ""

    def get_utc(self):
        return self.utc or ""

    def get_postal_code(self):
        return self.postal_code or ""

    def get_suggestions(self):
        return self.suggestions or ""

    def get_time_zone_code(self):
        return self.time_zone_code or ""

    def get_time_zone_name(self):
        return self.time_zone_name or ""



from .ResponseBase import ResponseBase

class PersonatorConsumerResponse(ResponseBase):
    def __init__(self, version="", transmission_reference="", transmission_results="", total_records="", records=None):
        self.version = version
        self.transmission_reference = transmission_reference
        self.transmission_results = transmission_results
        self.total_records = total_records
        self.records = records if records is not None else []

    @classmethod
    def populate_from_dict(cls, data: dict):
        records = [PersonatorConsumerRecord.from_dict(record) for record in data.get("Records", [])]
        return cls(
            version=data.get("Version", ""),
            transmission_reference=data.get("TransmissionReference", ""),
            transmission_results=data.get("TransmissionResults", ""),
            total_records = data.get("TotalRecords", ""),
            records=records
        )
    
    
    # Setters
    def set_version(self, version):
        self.version = version

    def set_transmission_reference(self, transmission_reference):
        self.transmission_reference = transmission_reference

    def set_transmission_results(self, transmission_results):
        self.transmission_results = transmission_results

    def set_total_records(self, total_records):
        self.total_records = total_records

    # Getters
    def get_version(self):
        return self.version or ""

    def get_transmission_reference(self):
        return self.transmission_reference or ""

    def get_transmission_results(self):
        return self.transmission_results or ""

    def get_total_records(self):
        return self.total_records or ""
    
class PersonatorConsumerRecord(ResponseBase):
    def __init__(self, address_delivery_installation="", address_extras="", address_house_number="", address_key="",
                 address_line_1="", address_line_2="", address_lock_box="", address_post_direction="", 
                 address_pre_direction="", address_private_mailbox_name="", address_private_mailbox_range="",
                 address_route_service="", address_street_name="", address_street_suffix="", address_suite_name="",
                 address_suite_number="", address_type_code="", area_code="", cbsacode="", cbsadivision_code="",
                 cbsadivision_level="", cbsadivision_title="", cbsalevel="", cbsatitle="", carrier_route="",
                 census_block="", census_key="", census_tract="", children_age_range="", city="", city_abbreviation="",
                 company_name="", congressional_district="", country_code="", country_name="", county_fips="",
                 county_name="", county_subdivision_code="", county_subdivision_name="", credit_card_user="",
                 date_last_confirmed="", date_of_birth="", date_of_death="", delivery_indicator="", 
                 delivery_point_check_digit="", delivery_point_code="", demographics_gender="", demographics_results="",
                 distance_address_to_ip="", domain_name="", education="", elementary_school_district_code="",
                 elementary_school_district_name="", email_address="", estimated_home_value="", ethnic_code="", 
                 ethnic_group="", gender="", gender2="", household_income="", household_size="", ip_address="",
                 ip_city="", ip_connection_speed="", ip_connection_type="", ip_continent="", 
                 ip_country_abbreviation="", ip_country_name="", ip_domain_name="", ip_isp_name="", 
                 ip_latitude="", ip_longitude="", ip_postal_code="", ip_proxy_description="", ip_proxy_type="",
                 ip_region="", ip_utc="", latitude="", length_of_residence="", longitude="", mailbox_name="",
                 marital_status="", melissa_address_key="", melissa_address_key_base="", melissa_identity_key="",
                 move_date="", name_first="", name_first2="", name_full="", name_last="", name_last2="", name_middle="",
                 name_middle2="", name_prefix="", name_prefix2="", name_suffix="", name_suffix2="", new_area_code="",
                 occupation="", own_rent="", phone_country_code="", phone_country_name="", phone_extension="",
                 phone_number="", phone_prefix="", phone_suffix="", place_code="", place_name="", plus4="",
                 political_party="", postal_code="", presence_of_children="", presence_of_senior="", private_mail_box="",
                 record_id="", results="", salutation="", secondary_school_district_code="", 
                 secondary_school_district_name="", state="", state_district_lower="", state_district_upper="",
                 state_name="", suite="", top_level_domain="", utc="", unified_school_district_code="",
                 unified_school_district_name="", record_extras="", reserved=""):
        self.address_delivery_installation = address_delivery_installation
        self.address_extras = address_extras
        self.address_house_number = address_house_number
        self.address_key = address_key
        self.address_line_1 = address_line_1
        self.address_line_2 = address_line_2
        self.address_lock_box = address_lock_box
        self.address_post_direction = address_post_direction
        self.address_pre_direction = address_pre_direction
        self.address_private_mailbox_name = address_private_mailbox_name
        self.address_private_mailbox_range = address_private_mailbox_range
        self.address_route_service = address_route_service
        self.address_street_name = address_street_name
        self.address_street_suffix = address_street_suffix
        self.address_suite_name = address_suite_name
        self.address_suite_number = address_suite_number
        self.address_type_code = address_type_code
        self.area_code = area_code
        self.cbsacode = cbsacode
        self.cbsadivision_code = cbsadivision_code
        self.cbsadivision_level = cbsadivision_level
        self.cbsadivision_title = cbsadivision_title
        self.cbsalevel = cbsalevel
        self.cbsatitle = cbsatitle
        self.carrier_route = carrier_route
        self.census_block = census_block
        self.census_key = census_key
        self.census_tract = census_tract
        self.children_age_range = children_age_range
        self.city = city
        self.city_abbreviation = city_abbreviation
        self.company_name = company_name
        self.congressional_district = congressional_district
        self.country_code = country_code
        self.country_name = country_name
        self.county_fips = county_fips
        self.county_name = county_name
        self.county_subdivision_code = county_subdivision_code
        self.county_subdivision_name = county_subdivision_name
        self.credit_card_user = credit_card_user
        self.date_last_confirmed = date_last_confirmed
        self.date_of_birth = date_of_birth
        self.date_of_death = date_of_death
        self.delivery_indicator = delivery_indicator
        self.delivery_point_check_digit = delivery_point_check_digit
        self.delivery_point_code = delivery_point_code
        self.demographics_gender = demographics_gender
        self.demographics_results = demographics_results
        self.distance_address_to_ip = distance_address_to_ip
        self.domain_name = domain_name
        self.education = education
        self.elementary_school_district_code = elementary_school_district_code
        self.elementary_school_district_name = elementary_school_district_name
        self.email_address = email_address
        self.estimated_home_value = estimated_home_value
        self.ethnic_code = ethnic_code
        self.ethnic_group = ethnic_group
        self.gender = gender
        self.gender2 = gender2
        self.household_income = household_income
        self.household_size = household_size
        self.ip_address = ip_address
        self.ip_city = ip_city
        self.ip_connection_speed = ip_connection_speed
        self.ip_connection_type = ip_connection_type
        self.ip_continent = ip_continent
        self.ip_country_abbreviation = ip_country_abbreviation
        self.ip_country_name = ip_country_name
        self.ip_domain_name = ip_domain_name
        self.ip_isp_name = ip_isp_name
        self.ip_latitude = ip_latitude
        self.ip_longitude = ip_longitude
        self.ip_postal_code = ip_postal_code
        self.ip_proxy_description = ip_proxy_description
        self.ip_proxy_type = ip_proxy_type
        self.ip_region = ip_region
        self.ip_utc = ip_utc
        self.latitude = latitude
        self.length_of_residence = length_of_residence
        self.longitude = longitude
        self.mailbox_name = mailbox_name
        self.marital_status = marital_status
        self.melissa_address_key = melissa_address_key
        self.melissa_address_key_base = melissa_address_key_base
        self.melissa_identity_key = melissa_identity_key
        self.move_date = move_date
        self.name_first = name_first
        self.name_first2 = name_first2
        self.name_full = name_full
        self.name_last = name_last
        self.name_last2 = name_last2
        self.name_middle = name_middle
        self.name_middle2 = name_middle2
        self.name_prefix = name_prefix
        self.name_prefix2 = name_prefix2
        self.name_suffix = name_suffix
        self.name_suffix2 = name_suffix2
        self.new_area_code = new_area_code
        self.occupation = occupation
        self.own_rent = own_rent
        self.phone_country_code = phone_country_code
        self.phone_country_name = phone_country_name
        self.phone_extension = phone_extension
        self.phone_number = phone_number
        self.phone_prefix = phone_prefix
        self.phone_suffix = phone_suffix
        self.place_code = place_code
        self.place_name = place_name
        self.plus4 = plus4
        self.political_party = political_party
        self.postal_code = postal_code
        self.presence_of_children = presence_of_children
        self.presence_of_senior = presence_of_senior
        self.private_mail_box = private_mail_box
        self.record_id = record_id
        self.results = results
        self.salutation = salutation
        self.secondary_school_district_code = secondary_school_district_code
        self.secondary_school_district_name = secondary_school_district_name
        self.state = state
        self.state_district_lower = state_district_lower
        self.state_district_upper = state_district_upper
        self.state_name = state_name
        self.suite = suite
        self.top_level_domain = top_level_domain
        self.utc = utc
        self.unified_school_district_code = unified_school_district_code
        self.unified_school_district_name = unified_school_district_name
        self.record_extras = record_extras
        self.reserved = reserved

    @classmethod
    def from_dict(cls, data):
        cls.data = data
        return cls(
            address_delivery_installation=data.get("AddressDeliveryInstallation", ""),
            address_extras=data.get("AddressExtras", ""),
            address_house_number=data.get("AddressHouseNumber", ""),
            address_key=data.get("AddressKey", ""),
            address_line_1=data.get("AddressLine1", ""),
            address_line_2=data.get("AddressLine2", ""),
            address_lock_box=data.get("AddressLockBox", ""),
            address_post_direction=data.get("AddressPostDirection", ""),
            address_pre_direction=data.get("AddressPreDirection", ""),
            address_private_mailbox_name=data.get("AddressPrivateMailboxName", ""),
            address_private_mailbox_range=data.get("AddressPrivateMailboxRange", ""),
            address_route_service=data.get("AddressRouteService", ""),
            address_street_name=data.get("AddressStreetName", ""),
            address_street_suffix=data.get("AddressStreetSuffix", ""),
            address_suite_name=data.get("AddressSuiteName", ""),
            address_suite_number=data.get("AddressSuiteNumber", ""),
            address_type_code=data.get("AddressTypeCode", ""),
            area_code=data.get("AreaCode", ""),
            cbsacode=data.get("CBSACode", ""),
            cbsadivision_code=data.get("CBSADivisionCode", ""),
            cbsadivision_level=data.get("CBSADivisionLevel", ""),
            cbsadivision_title=data.get("CBSADivisionTitle", ""),
            cbsalevel=data.get("CBSALevel", ""),
            cbsatitle=data.get("CBSATitle", ""),
            carrier_route=data.get("CarrierRoute", ""),
            census_block=data.get("CensusBlock", ""),
            census_key=data.get("CensusKey", ""),
            census_tract=data.get("CensusTract", ""),
            children_age_range=data.get("ChildrenAgeRange", ""),
            city=data.get("City", ""),
            city_abbreviation=data.get("CityAbbreviation", ""),
            company_name=data.get("CompanyName", ""),
            congressional_district=data.get("CongressionalDistrict", ""),
            country_code=data.get("CountryCode", ""),
            country_name=data.get("CountryName", ""),
            county_fips=data.get("CountyFIPS", ""),
            county_name=data.get("CountyName", ""),
            county_subdivision_code=data.get("CountySubdivisionCode", ""),
            county_subdivision_name=data.get("CountySubdivisionName", ""),
            credit_card_user=data.get("CreditCardUser", ""),
            date_last_confirmed=data.get("DateLastConfirmed", ""),
            date_of_birth=data.get("DateOfBirth", ""),
            date_of_death=data.get("DateOfDeath", ""),
            delivery_indicator=data.get("DeliveryIndicator", ""),
            delivery_point_check_digit=data.get("DeliveryPointCheckDigit", ""),
            delivery_point_code=data.get("DeliveryPointCode", ""),
            demographics_gender=data.get("DemographicsGender", ""),
            demographics_results=data.get("DemographicsResults", ""),
            distance_address_to_ip=data.get("DistanceAddressToIP", ""),
            domain_name=data.get("DomainName", ""),
            education=data.get("Education", ""),
            elementary_school_district_code=data.get("ElementarySchoolDistrictCode", ""),
            elementary_school_district_name=data.get("ElementarySchoolDistrictName", ""),
            email_address=data.get("EmailAddress", ""),
            estimated_home_value=data.get("EstimatedHomeValue", ""),
            ethnic_code=data.get("EthnicCode", ""),
            ethnic_group=data.get("EthnicGroup", ""),
            gender=data.get("Gender", ""),
            gender2=data.get("Gender2", ""),
            household_income=data.get("HouseholdIncome", ""),
            household_size=data.get("HouseholdSize", ""),
            ip_address=data.get("IPAddress", ""),
            ip_city=data.get("IPCity", ""),
            ip_connection_speed=data.get("IPConnectionSpeed", ""),
            ip_connection_type=data.get("IPConnectionType", ""),
            ip_continent=data.get("IPContinent", ""),
            ip_country_abbreviation=data.get("IPCountryAbbreviation", ""),
            ip_country_name=data.get("IPCountryName", ""),
            ip_domain_name=data.get("IPDomainName", ""),
            ip_isp_name=data.get("IPISPName", ""),
            ip_latitude=data.get("IPLatitude", ""),
            ip_longitude=data.get("IPLongitude", ""),
            ip_postal_code=data.get("IPPostalCode", ""),
            ip_proxy_description=data.get("IPProxyDescription", ""),
            ip_proxy_type=data.get("IPProxyType", ""),
            ip_region=data.get("IPRegion", ""),
            ip_utc=data.get("IPUTC", ""),
            latitude=data.get("Latitude", ""),
            length_of_residence=data.get("LengthOfResidence", ""),
            longitude=data.get("Longitude", ""),
            mailbox_name=data.get("MailboxName", ""),
            marital_status=data.get("MaritalStatus", ""),
            melissa_address_key=data.get("MelissaAddressKey", ""),
            melissa_address_key_base=data.get("MelissaAddressKeyBase", ""),
            melissa_identity_key=data.get("MelissaIdentityKey", ""),
            move_date=data.get("MoveDate", ""),
            name_first=data.get("NameFirst", ""),
            name_first2=data.get("NameFirst2", ""),
            name_full=data.get("NameFull", ""),
            name_last=data.get("NameLast", ""),
            name_last2=data.get("NameLast2", ""),
            name_middle=data.get("NameMiddle", ""),
            name_middle2=data.get("NameMiddle2", ""),
            name_prefix=data.get("NamePrefix", ""),
            name_prefix2=data.get("NamePrefix2", ""),
            name_suffix=data.get("NameSuffix", ""),
            name_suffix2=data.get("NameSuffix2", ""),
            new_area_code=data.get("NewAreaCode", ""),
            occupation=data.get("Occupation", ""),
            own_rent=data.get("OwnRent", ""),
            phone_country_code=data.get("PhoneCountryCode", ""),
            phone_country_name=data.get("PhoneCountryName", ""),
            phone_extension=data.get("PhoneExtension", ""),
            phone_number=data.get("PhoneNumber", ""),
            phone_prefix=data.get("PhonePrefix", ""),
            phone_suffix=data.get("PhoneSuffix", ""),
            place_code=data.get("PlaceCode", ""),
            place_name=data.get("PlaceName", ""),
            plus4=data.get("Plus4", ""),
            political_party=data.get("PoliticalParty", ""),
            postal_code=data.get("PostalCode", ""),
            presence_of_children=data.get("PresenceOfChildren", ""),
            presence_of_senior=data.get("PresenceOfSenior", ""),
            private_mail_box=data.get("PrivateMailBox", ""),
            record_id=data.get("RecordID", ""),
            results=data.get("Results", ""),
            salutation=data.get("Salutation", ""),
            secondary_school_district_code=data.get("SecondarySchoolDistrictCode", ""),
            secondary_school_district_name=data.get("SecondarySchoolDistrictName", ""),
            state=data.get("State", ""),
            state_district_lower=data.get("StateDistrictLower", ""),
            state_district_upper=data.get("StateDistrictUpper", ""),
            state_name=data.get("StateName", ""),
            suite=data.get("Suite", ""),
            top_level_domain=data.get("TopLevelDomain", ""),
            utc=data.get("UTC", ""),
            unified_school_district_code=data.get("UnifiedSchoolDistrictCode", ""),
            unified_school_district_name=data.get("UnifiedSchoolDistrictName", ""),
            record_extras=data.get("RecordExtras", ""),
            reserved=data.get("Reserved", "")
        )
    
    # Setters

    def set_address_delivery_installation(self, address_delivery_installation):
        self.address_delivery_installation = address_delivery_installation

    def set_address_extras(self, address_extras):
        self.address_extras = address_extras

    def set_address_house_number(self, address_house_number):
        self.address_house_number = address_house_number

    def set_address_key(self, address_key):
        self.address_key = address_key

    def set_address_line_1(self, address_line_1):
        self.address_line_1 = address_line_1

    def set_address_line_2(self, address_line_2):
        self.address_line_2 = address_line_2

    def set_address_lock_box(self, address_lock_box):
        self.address_lock_box = address_lock_box

    def set_address_post_direction(self, address_post_direction):
        self.address_post_direction = address_post_direction

    def set_address_pre_direction(self, address_pre_direction):
        self.address_pre_direction = address_pre_direction

    def set_address_private_mailbox_name(self, address_private_mailbox_name):
        self.address_private_mailbox_name = address_private_mailbox_name

    def set_address_private_mailbox_range(self, address_private_mailbox_range):
        self.address_private_mailbox_range = address_private_mailbox_range

    def set_address_route_service(self, address_route_service):
        self.address_route_service = address_route_service

    def set_address_street_name(self, address_street_name):
        self.address_street_name = address_street_name

    def set_address_street_suffix(self, address_street_suffix):
        self.address_street_suffix = address_street_suffix

    def set_address_suite_name(self, address_suite_name):
        self.address_suite_name = address_suite_name

    def set_address_suite_number(self, address_suite_number):
        self.address_suite_number = address_suite_number

    def set_address_type_code(self, address_type_code):
        self.address_type_code = address_type_code

    def set_area_code(self, area_code):
        self.area_code = area_code

    def set_cbsa_code(self, cbsa_code):
        self.cbsa_code = cbsa_code

    def set_cbsa_division_code(self, cbsa_division_code):
        self.cbsa_division_code = cbsa_division_code

    def set_cbsa_division_level(self, cbsa_division_level):
        self.cbsa_division_level = cbsa_division_level

    def set_cbsa_division_title(self, cbsa_division_title):
        self.cbsa_division_title = cbsa_division_title

    def set_cbsa_level(self, cbsa_level):
        self.cbsa_level = cbsa_level

    def set_cbsa_title(self, cbsa_title):
        self.cbsa_title = cbsa_title

    def set_carrier_route(self, carrier_route):
        self.carrier_route = carrier_route

    def set_census_block(self, census_block):
        self.census_block = census_block

    def set_census_key(self, census_key):
        self.census_key = census_key

    def set_census_tract(self, census_tract):
        self.census_tract = census_tract

    def set_children_age_range(self, children_age_range):
        self.children_age_range = children_age_range

    def set_city(self, city):
        self.city = city

    def set_city_abbreviation(self, city_abbreviation):
        self.city_abbreviation = city_abbreviation

    def set_company_name(self, company_name):
        self.company_name = company_name

    def set_congressional_district(self, congressional_district):
        self.congressional_district = congressional_district

    def set_country_code(self, country_code):
        self.country_code = country_code

    def set_country_name(self, country_name):
        self.country_name = country_name

    def set_county_fips(self, county_fips):
        self.county_fips = county_fips

    def set_county_name(self, county_name):
        self.county_name = county_name

    def set_county_subdivision_code(self, county_subdivision_code):
        self.county_subdivision_code = county_subdivision_code

    def set_county_subdivision_name(self, county_subdivision_name):
        self.county_subdivision_name = county_subdivision_name

    def set_credit_card_user(self, credit_card_user):
        self.credit_card_user = credit_card_user

    def set_date_last_confirmed(self, date_last_confirmed):
        self.date_last_confirmed = date_last_confirmed

    def set_date_of_birth(self, date_of_birth):
        self.date_of_birth = date_of_birth

    def set_date_of_death(self, date_of_death):
        self.date_of_death = date_of_death

    def set_delivery_indicator(self, delivery_indicator):
        self.delivery_indicator = delivery_indicator

    def set_delivery_point_check_digit(self, delivery_point_check_digit):
        self.delivery_point_check_digit = delivery_point_check_digit

    def set_delivery_point_code(self, delivery_point_code):
        self.delivery_point_code = delivery_point_code

    def set_demographics_gender(self, demographics_gender):
        self.demographics_gender = demographics_gender

    def set_demographics_results(self, demographics_results):
        self.demographics_results = demographics_results

    def set_distance_address_to_ip(self, distance_address_to_ip):
        self.distance_address_to_ip = distance_address_to_ip

    def set_domain_name(self, domain_name):
        self.domain_name = domain_name

    def set_education(self, education):
        self.education = education

    def set_elementary_school_district_code(self, elementary_school_district_code):
        self.elementary_school_district_code = elementary_school_district_code

    def set_elementary_school_district_name(self, elementary_school_district_name):
        self.elementary_school_district_name = elementary_school_district_name

    def set_email_address(self, email_address):
        self.email_address = email_address

    def set_estimated_home_value(self, estimated_home_value):
        self.estimated_home_value = estimated_home_value

    def set_ethnic_code(self, ethnic_code):
        self.ethnic_code = ethnic_code

    def set_ethnic_group(self, ethnic_group):
        self.ethnic_group = ethnic_group

    def set_gender(self, gender):
        self.gender = gender

    def set_gender2(self, gender2):
        self.gender2 = gender2

    def set_household_income(self, household_income):
        self.household_income = household_income

    def set_household_size(self, household_size):
        self.household_size = household_size

    def set_ip_address(self, ip_address):
        self.ip_address = ip_address

    def set_ip_city(self, ip_city):
        self.ip_city = ip_city

    def set_ip_connection_speed(self, ip_connection_speed):
        self.ip_connection_speed = ip_connection_speed

    def set_ip_connection_type(self, ip_connection_type):
        self.ip_connection_type = ip_connection_type

    def set_ip_continent(self, ip_continent):
        self.ip_continent = ip_continent

    def set_ip_country_abbreviation(self, ip_country_abbreviation):
        self.ip_country_abbreviation = ip_country_abbreviation

    def set_ip_country_name(self, ip_country_name):
        self.ip_country_name = ip_country_name

    def set_ip_domain_name(self, ip_domain_name):
        self.ip_domain_name = ip_domain_name

    def set_ip_isp_name(self, ip_isp_name):
        self.ip_isp_name = ip_isp_name

    def set_ip_latitude(self, ip_latitude):
        self.ip_latitude = ip_latitude

    def set_ip_longitude(self, ip_longitude):
        self.ip_longitude = ip_longitude

    def set_ip_postal_code(self, ip_postal_code):
        self.ip_postal_code = ip_postal_code

    def set_ip_proxy_description(self, ip_proxy_description):
        self.ip_proxy_description = ip_proxy_description

    def set_ip_proxy_type(self, ip_proxy_type):
        self.ip_proxy_type = ip_proxy_type

    def set_ip_region(self, ip_region):
        self.ip_region = ip_region

    def set_ip_utc(self, ip_utc):
        self.ip_utc = ip_utc

    def set_latitude(self, latitude):
        self.latitude = latitude

    def set_length_of_residence(self, length_of_residence):
        self.length_of_residence = length_of_residence

    def set_longitude(self, longitude):
        self.longitude = longitude

    def set_mailbox_name(self, mailbox_name):
        self.mailbox_name = mailbox_name

    def set_marital_status(self, marital_status):
        self.marital_status = marital_status

    def set_melissa_address_key(self, melissa_address_key):
        self.melissa_address_key = melissa_address_key

    def set_melissa_address_key_base(self, melissa_address_key_base):
        self.melissa_address_key_base = melissa_address_key_base

    def set_melissa_identity_key(self, melissa_identity_key):
        self.melissa_identity_key = melissa_identity_key

    def set_move_date(self, move_date):
        self.move_date = move_date

    def set_name_first(self, name_first):
        self.name_first = name_first

    def set_name_first2(self, name_first2):
        self.name_first2 = name_first2

    def set_name_full(self, name_full):
        self.name_full = name_full

    def set_name_last(self, name_last):
        self.name_last = name_last

    def set_name_last2(self, name_last2):
        self.name_last2 = name_last2

    def set_name_middle(self, name_middle):
        self.name_middle = name_middle

    def set_name_middle2(self, name_middle2):
        self.name_middle2 = name_middle2

    def set_name_prefix(self, name_prefix):
        self.name_prefix = name_prefix

    def set_name_prefix2(self, name_prefix2):
        self.name_prefix2 = name_prefix2

    def set_name_suffix(self, name_suffix):
        self.name_suffix = name_suffix

    def set_name_suffix2(self, name_suffix2):
        self.name_suffix2 = name_suffix2

    def set_new_area_code(self, new_area_code):
        self.new_area_code = new_area_code

    def set_occupation(self, occupation):
        self.occupation = occupation

    def set_own_rent(self, own_rent):
        self.own_rent = own_rent

    def set_phone_country_code(self, phone_country_code):
        self.phone_country_code = phone_country_code

    def set_phone_country_name(self, phone_country_name):
        self.phone_country_name = phone_country_name

    def set_phone_extension(self, phone_extension):
        self.phone_extension = phone_extension

    def set_phone_number(self, phone_number):
        self.phone_number = phone_number

    def set_phone_prefix(self, phone_prefix):
        self.phone_prefix = phone_prefix

    def set_phone_suffix(self, phone_suffix):
        self.phone_suffix = phone_suffix

    def set_place_code(self, place_code):
        self.place_code = place_code

    def set_place_name(self, place_name):
        self.place_name = place_name

    def set_plus4(self, plus4):
        self.plus4 = plus4

    def set_political_party(self, political_party):
        self.political_party = political_party

    def set_postal_code(self, postal_code):
        self.postal_code = postal_code

    def set_presence_of_children(self, presence_of_children):
        self.presence_of_children = presence_of_children

    def set_presence_of_senior(self, presence_of_senior):
        self.presence_of_senior = presence_of_senior

    def set_private_mail_box(self, private_mail_box):
        self.private_mail_box = private_mail_box

    def set_record_id(self, record_id):
        self.record_id = record_id

    def set_results(self, results):
        self.results = results

    def set_salutation(self, salutation):
        self.salutation = salutation

    def set_secondary_school_district_code(self, secondary_school_district_code):
        self.secondary_school_district_code = secondary_school_district_code

    def set_secondary_school_district_name(self, secondary_school_district_name):
        self.secondary_school_district_name = secondary_school_district_name

    def set_state(self, state):
        self.state = state

    def set_state_district_lower(self, state_district_lower):
        self.state_district_lower = state_district_lower

    def set_state_district_upper(self, state_district_upper):
        self.state_district_upper = state_district_upper

    def set_state_name(self, state_name):
        self.state_name = state_name

    def set_suite(self, suite):
        self.suite = suite

    def set_top_level_domain(self, top_level_domain):
        self.top_level_domain = top_level_domain

    def set_utc(self, utc):
        self.utc = utc

    def set_unified_school_district_code(self, unified_school_district_code):
        self.unified_school_district_code = unified_school_district_code

    def set_unified_school_district_name(self, unified_school_district_name):
        self.unified_school_district_name = unified_school_district_name

    def set_record_extras(self, record_extras):
        self.record_extras = record_extras

    def set_reserved(self, reserved):
        self.reserved = reserved

    # Getters

    def get_address_delivery_installation(self):
        return self.address_delivery_installation or ""

    def get_address_extras(self):
        return self.address_extras or ""

    def get_address_house_number(self):
        return self.address_house_number or ""

    def get_address_key(self):
        return self.address_key or ""

    def get_address_line_1(self):
        return self.address_line_1 or ""

    def get_address_line_2(self):
        return self.address_line_2 or ""

    def get_address_lock_box(self):
        return self.address_lock_box or ""

    def get_address_post_direction(self):
        return self.address_post_direction or ""

    def get_address_pre_direction(self):
        return self.address_pre_direction or ""

    def get_address_private_mailbox_name(self):
        return self.address_private_mailbox_name or ""

    def get_address_private_mailbox_range(self):
        return self.address_private_mailbox_range or ""

    def get_address_route_service(self):
        return self.address_route_service or ""

    def get_address_street_name(self):
        return self.address_street_name or ""

    def get_address_street_suffix(self):
        return self.address_street_suffix or ""

    def get_address_suite_name(self):
        return self.address_suite_name or ""

    def get_address_suite_number(self):
        return self.address_suite_number or ""

    def get_address_type_code(self):
        return self.address_type_code or ""

    def get_area_code(self):
        return self.area_code or ""

    def get_cbsa_code(self):
        return self.cbsa_code or ""

    def get_cbsa_division_code(self):
        return self.cbsa_division_code or ""

    def get_cbsa_division_level(self):
        return self.cbsa_division_level or ""

    def get_cbsa_division_title(self):
        return self.cbsa_division_title or ""

    def get_cbsa_level(self):
        return self.cbsa_level or ""

    def get_cbsa_title(self):
        return self.cbsa_title or ""

    def get_carrier_route(self):
        return self.carrier_route or ""

    def get_census_block(self):
        return self.census_block or ""

    def get_census_key(self):
        return self.census_key or ""

    def get_census_tract(self):
        return self.census_tract or ""

    def get_children_age_range(self):
        return self.children_age_range or ""

    def get_city(self):
        return self.city or ""

    def get_city_abbreviation(self):
        return self.city_abbreviation or ""

    def get_company_name(self):
        return self.company_name or ""

    def get_congressional_district(self):
        return self.congressional_district or ""

    def get_country_code(self):
        return self.country_code or ""

    def get_country_name(self):
        return self.country_name or ""

    def get_county_fips(self):
        return self.county_fips or ""

    def get_county_name(self):
        return self.county_name or ""

    def get_county_subdivision_code(self):
        return self.county_subdivision_code or ""

    def get_county_subdivision_name(self):
        return self.county_subdivision_name or ""

    def get_credit_card_user(self):
        return self.credit_card_user or ""

    def get_date_last_confirmed(self):
        return self.date_last_confirmed or ""

    def get_date_of_birth(self):
        return self.date_of_birth or ""

    def get_date_of_death(self):
        return self.date_of_death or ""

    def get_delivery_indicator(self):
        return self.delivery_indicator or ""

    def get_delivery_point_check_digit(self):
        return self.delivery_point_check_digit or ""

    def get_delivery_point_code(self):
        return self.delivery_point_code or ""

    def get_demographics_gender(self):
        return self.demographics_gender or ""

    def get_demographics_results(self):
        return self.demographics_results or ""

    def get_distance_address_to_ip(self):
        return self.distance_address_to_ip or ""

    def get_domain_name(self):
        return self.domain_name or ""

    def get_education(self):
        return self.education or ""

    def get_elementary_school_district_code(self):
        return self.elementary_school_district_code or ""

    def get_elementary_school_district_name(self):
        return self.elementary_school_district_name or ""

    def get_email_address(self):
        return self.email_address or ""

    def get_estimated_home_value(self):
        return self.estimated_home_value or ""

    def get_ethnic_code(self):
        return self.ethnic_code or ""

    def get_ethnic_group(self):
        return self.ethnic_group or ""

    def get_gender(self):
        return self.gender or ""

    def get_gender2(self):
        return self.gender2 or ""

    def get_household_income(self):
        return self.household_income or ""

    def get_household_size(self):
        return self.household_size or ""

    def get_ip_address(self):
        return self.ip_address or ""

    def get_ip_city(self):
        return self.ip_city or ""

    def get_ip_connection_speed(self):
        return self.ip_connection_speed or ""

    def get_ip_connection_type(self):
        return self.ip_connection_type or ""

    def get_ip_continent(self):
        return self.ip_continent or ""

    def get_ip_country_abbreviation(self):
        return self.ip_country_abbreviation or ""

    def get_ip_country_name(self):
        return self.ip_country_name or ""

    def get_ip_domain_name(self):
        return self.ip_domain_name or ""

    def get_ip_isp_name(self):
        return self.ip_isp_name or ""

    def get_ip_latitude(self):
        return self.ip_latitude or ""

    def get_ip_longitude(self):
        return self.ip_longitude or ""

    def get_ip_postal_code(self):
        return self.ip_postal_code or ""

    def get_ip_proxy_description(self):
        return self.ip_proxy_description or ""

    def get_ip_proxy_type(self):
        return self.ip_proxy_type or ""

    def get_ip_region(self):
        return self.ip_region or ""

    def get_ip_utc(self):
        return self.ip_utc or ""

    def get_latitude(self):
        return self.latitude or ""

    def get_length_of_residence(self):
        return self.length_of_residence or ""

    def get_longitude(self):
        return self.longitude or ""

    def get_mailbox_name(self):
        return self.mailbox_name or ""

    def get_marital_status(self):
        return self.marital_status or ""

    def get_melissa_address_key(self):
        return self.melissa_address_key or ""

    def get_melissa_address_key_base(self):
        return self.melissa_address_key_base or ""

    def get_melissa_identity_key(self):
        return self.melissa_identity_key or ""

    def get_move_date(self):
        return self.move_date or ""

    def get_name_first(self):
        return self.name_first or ""

    def get_name_first2(self):
        return self.name_first2 or ""

    def get_name_full(self):
        return self.name_full or ""

    def get_name_last(self):
        return self.name_last or ""

    def get_name_last2(self):
        return self.name_last2 or ""

    def get_name_middle(self):
        return self.name_middle or ""

    def get_name_middle2(self):
        return self.name_middle2 or ""

    def get_name_prefix(self):
        return self.name_prefix or ""

    def get_name_prefix2(self):
        return self.name_prefix2 or ""

    def get_name_suffix(self):
        return self.name_suffix or ""

    def get_name_suffix2(self):
        return self.name_suffix2 or ""

    def get_new_area_code(self):
        return self.new_area_code or ""

    def get_occupation(self):
        return self.occupation or ""

    def get_own_rent(self):
        return self.own_rent or ""

    def get_phone_country_code(self):
        return self.phone_country_code or ""

    def get_phone_country_name(self):
        return self.phone_country_name or ""

    def get_phone_extension(self):
        return self.phone_extension or ""

    def get_phone_number(self):
        return self.phone_number or ""

    def get_phone_prefix(self):
        return self.phone_prefix or ""

    def get_phone_suffix(self):
        return self.phone_suffix or ""

    def get_place_code(self):
        return self.place_code or ""

    def get_place_name(self):
        return self.place_name or ""

    def get_plus4(self):
        return self.plus4 or ""

    def get_political_party(self):
        return self.political_party or ""

    def get_postal_code(self):
        return self.postal_code or ""

    def get_presence_of_children(self):
        return self.presence_of_children or ""

    def get_presence_of_senior(self):
        return self.presence_of_senior or ""

    def get_private_mail_box(self):
        return self.private_mail_box or ""

    def get_record_id(self):
        return self.record_id or ""

    def get_results(self):
        return self.results or ""

    def get_salutation(self):
        return self.salutation or ""

    def get_secondary_school_district_code(self):
        return self.secondary_school_district_code or ""

    def get_secondary_school_district_name(self):
        return self.secondary_school_district_name or ""

    def get_state(self):
        return self.state or ""

    def get_state_district_lower(self):
        return self.state_district_lower or ""

    def get_state_district_upper(self):
        return self.state_district_upper or ""

    def get_state_name(self):
        return self.state_name or ""

    def get_suite(self):
        return self.suite or ""

    def get_top_level_domain(self):
        return self.top_level_domain or ""

    def get_utc(self):
        return self.utc or ""

    def get_unified_school_district_code(self):
        return self.unified_school_district_code or ""

    def get_unified_school_district_name(self):
        return self.unified_school_district_name or ""

    def get_record_extras(self):
        return self.record_extras or ""

    def get_reserved(self):
        return self.reserved or ""


import json
import os
from .CloudApiBase import CloudApiBase

class ReverseGeoCoder(CloudApiBase):

    def __init__(self, license=None):
        super().__init__("https://reversegeo.melissadata.net", "/v3/web/ReverseGeoCode/doLookup")
        self.license = license or os.environ.get("MD_LICENSE")
        self.parameters["format"] = "json"
        self.post_body = None

        self.parameter_mappings = {
            "latitude": "lat",
            "longitude": "long",
            "max_records": "recs",
            "max_distance": "dist",
            "transmission_reference": "t",
            "opt": "opt"
        }

                # Initialize private backing fields
        for attr in self.parameter_mappings.keys():
            setattr(self, f"_{attr}", None)

    def _set_parameter(self, attr_name, value):
        """
        Helper method to set a private attribute and update the parameters dictionary.
        """
        setattr(self, f"_{attr_name}", value)
        self.parameters[self.parameter_mappings[attr_name]] = value

    # Properties
    @property
    def latitude(self):
        return self._latitude

    @latitude.setter
    def latitude(self, value):
        self._set_parameter("latitude", value)

    @property
    def longitude(self):
        return self._longitude

    @longitude.setter
    def longitude(self, value):
        self._set_parameter("longitude", value)

    @property
    def max_records(self):
        return self._max_records

    @max_records.setter
    def max_records(self, value):
        self._set_parameter("max_records", value)

    @property
    def max_distance(self):
        return self._max_distance

    @max_distance.setter
    def max_distance(self, value):
        self._set_parameter("max_distance", value)

    @property
    def transmission_reference(self):
        return self._transmission_reference

    @transmission_reference.setter
    def transmission_reference(self, value):
        self._set_parameter("transmission_reference", value)

    @property
    def opt(self):
        return self._opt

    @opt.setter
    def opt(self, value):
        self._set_parameter("opt", value)


    # Setters
    def set_latitude(self, latitude: str):
        self._set_parameter("latitude", latitude)

    def set_longitude(self, longitude: str):
        self._set_parameter("longitude", longitude)

    def set_max_records(self, records: str):
        self._set_parameter("max_records", records)

    def set_max_distance(self, distance: str):
        self._set_parameter("max_distance", distance)

    def set_transmission_reference(self, transmission_reference: str):
        self._set_parameter("transmission_reference", transmission_reference)

    def set_opt(self, opt: str):
        self._set_parameter("opt", opt)

    def set_post_body(self, post_body):
        self.post_body = post_body

    def set_value(self, parameter, value):
        parameter = parameter.strip() if parameter else None
        
        if parameter in self.parameter_mappings:
            parameter_key = self.parameter_mappings[parameter]
            
            property_name = next((key for key, val in self.parameter_mappings.items() 
                                  if val.lower() == parameter_key.lower()), None)
            
            if property_name:
                if hasattr(self, property_name):
                    setattr(self, property_name, value)
        else:
            # If not in derived class, go to base class implementation
            super().set_value(parameter, value)

    # Getters
    def get_latitude(self) -> str:
        return self.parameters.get("lat", "")

    def get_longitude(self) -> str:
        return self.parameters.get("long", "")

    def get_max_records(self) -> str:
        return self.parameters.get("recs", "")

    def get_max_distance(self) -> str:
        return self.parameters.get("dist", "")

    def get_transmission_reference(self) -> str:
        return self.parameters.get("t", "")

    def get_opt(self) -> str:
        return self.parameters.get("opt", "")

    def get_post_batch_body(self):
        return self.post_body
    
    # Class Methods
    def get_do_lookup(self, response_type):
        """
        This synchronous function sends the CloudAPI request using the /v3/web/ReverseGeoCode/doLookup endpoint
        """
        self.set_endpoint("/v3/web/ReverseGeoCode/doLookup")
        return self.get(response_type)

    
    def post_do_lookup(self, response_type):
        """
        This synchronous function sends a post request to the Cloud API using the /v3/web/ReverseGeoCode/doLookup endpoint
        """
        self.set_endpoint("/v3/web/ReverseGeoCode/doLookup")
        response = self.send_post_request(self.post_body)
        return self.handle_response(response, response_type)

    
    def get_do_lookup_postal_codes(self, response_type):
        """
        This synchronous function sends the CloudAPI request using the /V3/WEB/ReverseGeoCode/doLookupPostalCodes endpoint
        """
        self.set_endpoint("/V3/WEB/ReverseGeoCode/doLookupPostalCodes")
        return self.get(response_type)

    
    def post_do_lookup_postal_codes(self, response_type):
        """
        This synchronous function sends a post request to the Cloud API using the /V3/WEB/ReverseGeoCode/doLookupPostalCodes endpoint
        """
        self.set_endpoint("/V3/WEB/ReverseGeoCode/doLookupPostalCodes")
        response = self.send_post_request(self.post_body)
        return self.handle_response(response, response_type)


    def get_api_version(self):
        """
        Makes a synchronous getversion request and parses the response for the API version.

        Returns:
            The Cloud API version.
        """
        json_response = self.send_version_request()
        response_objectect = json.loads(json_response)
        version = response_objectect["Version"]
        return version



from .ResponseBase import ResponseBase

class StreetRouteResponse(ResponseBase):
    def __init__(self, version="", units="", transmission_reference="", transmission_result="", total_records="", 
                 record_id="", results="", travel_time="", total_driving_distance="", records=None):
        self.version = version,
        self.units = units,
        self.transmission_reference = transmission_reference
        self.transmission_result = transmission_result
        self.total_records = total_records,
        self.record_id = record_id,
        self.results = results,
        self.travel_time = travel_time,
        self.total_driving_distance = total_driving_distance,
        self.records = records if records is not None else []

    @classmethod
    def populate_from_dict(cls, data: dict):
        cls.data = data
        return cls(
            version=data.get("Version", ""),
            units=data.get("Units", ""),
            transmission_reference=data.get("TransmissionReference", ""),
            transmission_result=data.get("TransmissionResult", ""),
            total_records=data.get("TotalRecords", ""),
            record_id=data.get("RecordID", ""),
            results=data.get("Results", ""),
            travel_time=data.get("TravelTime", ""),
            total_driving_distance=data.get("TotalDrivingDistance", ""),
            records=[StreetRouteRecord.from_dict(record) for record in data.get("Records", [])]
        )
    
    # Setters
    def set_version(self, version):
        self.version = version

    def set_units(self, units):
        self.units = units

    def set_transmission_reference(self, transmission_reference):
        self.transmission_reference = transmission_reference

    def set_transmission_result(self, transmission_result):
        self.transmission_result = transmission_result

    def set_total_records(self, total_records):
        self.total_records = total_records

    def set_record_id(self, record_id):
        self.record_id = record_id

    def set_results(self, results):
        self.results = results

    def set_travel_time(self, travel_time):
        self.travel_time = travel_time

    def set_total_driving_distance(self, total_driving_distance):
        self.total_driving_distance = total_driving_distance
    
    # Getters
    def get_version(self):
        return self.version or ""

    def get_units(self):
        return self.units or ""

    def get_transmission_reference(self):
        return self.transmission_reference or ""

    def get_transmission_result(self):
        return self.transmission_result or ""

    def get_total_records(self):
        return self.total_records or ""

    def get_record_id(self):
        return self.record_id or ""

    def get_results(self):
        return self.results or ""

    def get_travel_time(self):
        return self.travel_time or ""

    def get_total_driving_distance(self):
        return self.total_driving_distance or ""
        
class StreetRouteRecord(ResponseBase):
    def __init__(self, record_id="", results="", travel_time="", total_driving_distance=""):
        self.record_id = record_id
        self.results = results
        self.travel_time = travel_time
        self.total_driving_distance = total_driving_distance

    @classmethod
    def from_dict(cls, data):
        return cls(
            record_id=data.get("RecordID", ""),
            results=data.get("Results", ""),
            travel_time=data.get("TravelTime", ""),
            total_driving_distance=data.get("TotalDrivingDistance", "")
        )
    
    # Setters
    def set_record_id(self, record_id):
        self.record_id = record_id

    def set_results(self, results):
        self.results = results

    def set_travel_time(self, travel_time):
        self.travel_time = travel_time

    def set_total_driving_distance(self, total_driving_distance):
        self.total_driving_distance = total_driving_distance

    # Getters
    def get_record_id(self):
        return self.record_id or ""

    def get_results(self):
        return self.results or ""

    def get_travel_time(self):
        return self.travel_time or ""

    def get_total_driving_distance(self):
        return self.total_driving_distance or ""




